# Copyright 2014
# The Cloudscaling Group, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import copy
import itertools
from unittest import mock

from cinderclient import client as cinderclient
import fixtures
from glanceclient import client as glanceclient
from novaclient import client as novaclient
from oslo_concurrency import lockutils
from oslo_config import fixture as config_fixture
from oslotest import base as test_base

import ec2api.api.apirequest
from ec2api.api import clients
from ec2api.api import ec2utils
from ec2api import config
from ec2api.db import migration
from ec2api.db.sqlalchemy import api as db_backend
from ec2api.tests.unit import fakes
from ec2api.tests.unit import matchers
from ec2api.tests.unit import tools
import ec2api.wsgi


DB_SCHEMA = None


config.parse_args([], default_config_files=[])
# TODO(ft): temporary solution to prevent discovery of Nova API version
# in clients.nova. This should be removed when all tests either use
# MockOSMixin (which should be improved in its turn) to mock novaclient,
# or mock clients.nova directly.
clients._nova_api_version = '2.3'


def skip_not_implemented(test_item):
    def decorator(test_item):
        test_item.skip('The feature is not yet implemented')
    return decorator


def create_context(is_os_admin=False):
    session = (mock.sentinel.admin_session
               if is_os_admin else
               mock.sentinel.session)
    session.get_endpoint = mock.Mock(name="get_endpoint")
    session.get_endpoint.return_value = 'v2'
    return ec2api.context.RequestContext(fakes.ID_OS_USER, fakes.ID_OS_PROJECT,
                                         is_os_admin=is_os_admin,
                                         session=session)


class MockOSMixin(object):

    def mock(self, object_name):
        patcher = mock.patch(object_name)
        result = patcher.start()
        self.addCleanup(patcher.stop)
        return result

    def mock_all_os(self):
        self.neutron = self.mock_neutron()
        self.nova, self.nova_admin = self.mock_nova()
        self.glance = self.mock_glance()
        self.cinder = self.mock_cinder()

        self.get_os_admin_context = self.mock(
            'ec2api.context.get_os_admin_context')
        self.get_os_admin_context.return_value = (
            create_context(is_os_admin=True))

    def mock_neutron(self):
        neutron_patcher = mock.patch('neutronclient.v2_0.client.Client',
                                     autospec=True)
        neutron = neutron_patcher.start().return_value
        self.addCleanup(neutron_patcher.stop)
        return neutron

    def mock_nova(self):
        # NOTE(ft): create an extra mock for Nova calls with an admin account.
        # Also make sure that the admin account is used only for this calls.
        # The special mock is needed to validate tested function to retrieve
        # appropriate data, as long as only calls with admin account return
        # some specific data.
        novaclient_spec = novaclient.Client('2')
        nova = mock.create_autospec(novaclient_spec)
        nova_admin = mock.create_autospec(novaclient_spec)
        nova_patcher = mock.patch('novaclient.client.Client')
        novaclient_getter = nova_patcher.start()
        self.addCleanup(nova_patcher.stop)
        novaclient_getter.side_effect = (
            lambda *args, **kwargs: (
                nova_admin
                if (kwargs.get('session') == mock.sentinel.admin_session) else
                nova
                if (kwargs.get('session') == mock.sentinel.session) else
                None))
        return nova, nova_admin

    def mock_glance(self):
        glance_patcher = mock.patch('glanceclient.client.Client')
        with mock.patch('glanceclient.v2.schemas.Controller'):
            glance = mock.create_autospec(glanceclient.Client(endpoint='v2'))
            glance_patcher.start().return_value = glance
        self.addCleanup(glance_patcher.stop)
        return glance

    def mock_cinder(self):
        cinder_patcher = mock.patch('cinderclient.client.Client')
        cinder = mock.create_autospec(cinderclient.Client('2'))
        cinder_patcher.start().return_value = cinder
        self.addCleanup(cinder_patcher.stop)
        return cinder


class MockDBMixin(object):

    def mock_db(self):
        db_api_patcher = mock.patch('ec2api.db.api.IMPL', autospec=db_backend)
        db_api = db_api_patcher.start()
        self.addCleanup(db_api_patcher.stop)

        def set_mock_items(*items):
            db_api.__db_items = copy.copy(items)
            db_api.get_items.side_effect = (
                tools.get_db_api_get_items(*db_api.__db_items))
            db_api.get_item_by_id.side_effect = (
                tools.get_db_api_get_item_by_id(*db_api.__db_items))
            db_api.get_items_by_ids.side_effect = (
                tools.get_db_api_get_items_by_ids(*db_api.__db_items))
            db_api.get_items_ids.side_effect = (
                tools.get_db_api_get_items_ids(*db_api.__db_items))

        def add_mock_items(*items):
            merged_items = items + tuple(item for item in db_api.__db_items
                                         if all(i['id'] != item['id']
                                                for i in items))
            db_api.set_mock_items(*merged_items)

        setattr(db_api, 'set_mock_items', set_mock_items)
        setattr(db_api, 'add_mock_items', add_mock_items)
        return db_api


class BaseTestCase(MockOSMixin, MockDBMixin, test_base.BaseTestCase):

    def setUp(self):
        super(BaseTestCase, self).setUp()
        self._conf = self.useFixture(config_fixture.Config())
        self.configure(fatal_exception_format_errors=True)
        lock_path = self.useFixture(fixtures.TempDir()).path
        self.fixture = self.useFixture(
            config_fixture.Config(lockutils.CONF))
        self.fixture.config(lock_path=lock_path,
                            group='oslo_concurrency')

    def configure(self, **kwargs):
        self._conf.config(**kwargs)


class ApiTestCase(BaseTestCase):

    ANY_EXECUTE_ERROR = object()

    def setUp(self):
        super(ApiTestCase, self).setUp()
        self.mock_all_os()
        self.db_api = self.mock_db()
        self.isotime = self.mock('ec2api.api.ec2utils.isotime')

    def execute(self, action, args):
        status_code, response = self._execute(action, args)
        self.assertEqual(200, status_code,
                         self._format_error_message(status_code, response))
        return response

    def assert_execution_error(self, error_code, action, args):
        status_code, response = self._execute(action, args)
        if error_code == self.ANY_EXECUTE_ERROR:
            self.assertLessEqual(400, status_code)
        else:
            self.assertEqual(400, status_code)
            self.assertEqual(error_code, response['Error']['Code'],
                             self._format_error_message(status_code, response))

    def assert_any_call(self, func, *args, **kwargs):
        calls = func.mock_calls
        for call in calls:
            call_args = call[1]
            if matchers.ListMatches(call_args, args, orderless_lists=True):
                return
        self.assertEqual(False, True)

    def set_mock_db_items(self, *items):
        self.db_api.set_mock_items(*items)

    def add_mock_db_items(self, *items):
        self.db_api.add_mock_items(*items)

    def check_filtering(self, operation, resultset_key, filters):
        for name, value in filters:
            resp = self.execute(operation,
                                {'Filter.1.Name': name,
                                 'Filter.1.Value.1': str(value)})
            self.assertTrue(resp[resultset_key] is not None and
                            len(resp[resultset_key]) > 0,
                            'Filter by %s does not work' % name)

            resp = self.execute(operation,
                                {'Filter.1.Name': name,
                                 'Filter.1.Value.1': 'dummy filter value'})
            self.assertTrue(resp[resultset_key] is None or
                            len(resp[resultset_key]) == 0)

    def check_tag_support(self, operation, resultset_key, sample_item_id,
                          id_key, item_kinds=[]):
        self.db_api.get_tags = tools.CopyingMock(
            return_value=[{'item_id': sample_item_id,
                           'key': 'fake_key',
                           'value': 'fake_value'}])
        ec2_tags = [{'key': 'fake_key',
                     'value': 'fake_value'}]

        resp = self.execute(operation, {})
        tag_found = False
        if type(resultset_key) is list:
            resp_items = itertools.chain(*(r[resultset_key[1]]
                                           for r in resp[resultset_key[0]]))
        else:
            resp_items = resp[resultset_key]
            resultset_key = [resultset_key]
        for resp_item in resp_items:
            if resp_item.get(id_key) == sample_item_id:
                self.assertIn('tagSet', resp_item)
                self.assertThat(resp_item['tagSet'],
                                matchers.ListMatches(ec2_tags))
                tag_found = True
            else:
                self.assertTrue('tagSet' not in resp_item or
                                resp_item['tagSet'] == [])
        self.assertTrue(tag_found)
        if not item_kinds:
            item_kinds = (ec2utils.get_ec2_id_kind(sample_item_id),)
        self.assertTrue(self.db_api.get_tags.call_count == 1 and
                        (self.db_api.get_tags.mock_calls[0] in
                         (mock.call(mock.ANY, item_kinds, set()),
                          mock.call(mock.ANY, item_kinds, None))))
        self.db_api.reset_mock()

        id_param = '%s%s.1' % (id_key[0].capitalize(), id_key[1:])
        resp = self.execute(operation, {id_param: sample_item_id})
        self.assertTrue(
            self.db_api.get_tags.call_count == 1 and
            (self.db_api.get_tags.mock_calls[0] in
             (mock.call(mock.ANY, item_kinds, set([sample_item_id])),
              mock.call(mock.ANY, item_kinds, [sample_item_id]))))

        self.check_filtering(
             operation, resultset_key[0],
             [('tag-key', 'fake_key'),
              ('tag-value', 'fake_value'),
              ('tag:fake_key', 'fake_value')])

    def _execute(self, action, args):
        ec2_request = ec2api.api.apirequest.APIRequest(action, 'fake_v1', args)
        ec2_context = create_context()
        environ = {'REQUEST_METHOD': 'FAKE',
                   'ec2.request': ec2_request,
                   'ec2api.context': ec2_context}
        request = ec2api.wsgi.Request(environ)
        response = request.send(ec2api.api.Executor())
        return (response.status_code,
                self._check_and_transform_response(response, action))

    def _check_and_transform_response(self, response, action):
        body = tools.parse_xml(response.body)
        if response.status_code == 200:
            action_tag = '%sResponse' % action
            self.assertIn(action_tag, body)
            body = body.pop(action_tag)
            self.assertIn('requestId', body)
            body.pop('requestId')
        else:
            self.assertIn('Response', body)
            body = body.pop('Response')
            self.assertIn('RequestID', body)
            body.pop('RequestID')
            self.assertEqual(1, len(body))
            self.assertIn('Errors', body)
            body = body.pop('Errors')
            self.assertEqual(1, len(body))
            self.assertIn('Error', body)
            self.assertEqual(2, len(body['Error']))
        return body

    def _format_error_message(self, status_code, response):
        if status_code >= 400:
            return '%s: %s' % (response['Error']['Code'],
                               response['Error']['Message'])
        else:
            return ''


class DbTestCase(BaseTestCase):

    def setUp(self):
        super(DbTestCase, self).setUp()
        self.configure(connection='sqlite://', group='database')
        self.configure(sqlite_synchronous=False, group='database')
        self._init_db_schema()
        engine = db_backend.get_engine()
        conn = engine.connect()
        conn.connection.executescript(DB_SCHEMA)
        self.addCleanup(engine.dispose)

    def _init_db_schema(self):
        global DB_SCHEMA
        if not DB_SCHEMA:

            engine = db_backend.get_engine()
            conn = engine.connect()
            migration.db_sync()
            DB_SCHEMA = "".join(line for line in conn.connection.iterdump())
            engine.dispose()



# Copyright 2014
# The Cloudscaling Group, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from ec2api.api import common
from ec2api.api import ec2utils
from ec2api.db import api as db_api
from ec2api import exception
from ec2api.i18n import _


"""Tag related API implementation
"""


Validator = common.Validator


RESOURCE_TYPES = {
    'dopt': 'dhcp-options',
    'ami': 'image',
    'aki': 'image',
    'ari': 'image',
    'cgw': 'customer-gateway',
    'i': 'instance',
    'igw': 'internet-gateway',
    'eni': 'network-interface',
    'rtb': 'route-table',
    'snap': 'snapshot',
    'subnet': 'subnet',
    'sg': 'security-group',
    'vgw': 'vpn-gateway',
    'vol': 'volume',
    'vpc': 'vpc',
    'vpn': 'vpn-connection',
}


def create_tags(context, resource_id, tag):
    reason = None
    for tag_pair in tag:
        if not tag_pair.get('key'):
            reason = _('Not empty key must be present')
        elif len(tag_pair['key']) > 127:
            reason = _('Tag key exceeds the maximum length of 127 characters')
        elif tag_pair['key'].startswith('aws:'):
            reason = _("Tag keys starting with 'aws:' are reserved for "
                       "internal use")
        elif 'value' not in tag_pair:
            reason = _('Value must be present')
        elif len(tag_pair['value']) > 255:
            reason = _('Tag value exceeds the maximum length of 255 '
                       'characters')
        if reason:
            raise exception.InvalidParameterValue(
                    parameter='Tag', value=str(tag_pair), reason=reason)

    for item_id in resource_id:
        kind = ec2utils.get_ec2_id_kind(item_id)
        if kind not in RESOURCE_TYPES:
            raise exception.InvalidID(id=item_id)
        # NOTE(ft): check items exist (excluding images because AWS allows to
        # create a tag with any image id)
        if kind not in ('ami', 'ari', 'aki'):
            ec2utils.get_db_item(context, item_id)

    tags = [dict(item_id=item_id,
                 key=tag_pair['key'],
                 value=tag_pair['value'])
            for item_id in resource_id
            for tag_pair in tag]

    db_api.add_tags(context, tags)
    return True


def delete_tags(context, resource_id, tag=None):
    db_api.delete_tags(context, resource_id, tag)
    return True


class TagDescriber(common.NonOpenstackItemsDescriber):

    SORT_KEY = 'key'
    FILTER_MAP = {'key': 'key',
                  'tag-key': 'key',
                  'resource-id': 'resourceId',
                  'resource-type': 'resourceType',
                  'value': 'value',
                  'tag-value': 'value'}

    def get_db_items(self):
        return db_api.get_tags(self.context)

    def format(self, item):
        return _format_tag(item)


def describe_tags(context, filter=None, max_results=None, next_token=None):
    tag_describer = TagDescriber()
    formatted_tags = tag_describer.describe(
        context, filter=filter, max_results=max_results, next_token=next_token)
    result = {'tagSet': formatted_tags}
    if tag_describer.next_token:
        result['nextToken'] = tag_describer.next_token
    return result


def _format_tag(tag):
    kind = ec2utils.get_ec2_id_kind(tag['item_id'])
    return {
        'resourceType': RESOURCE_TYPES.get(kind, kind),
        'resourceId': tag['item_id'],
        'key': tag['key'],
        'value': tag['value'],
    }



# Copyright 2014
# The Cloudscaling Group, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest import mock

from ec2api.tests.unit import base
from ec2api.tests.unit import fakes
from ec2api.tests.unit import matchers
from ec2api.tests.unit import tools


class VolumeTestCase(base.ApiTestCase):

    def test_describe_volumes(self):
        self.cinder.volumes.list.return_value = [
            fakes.OSVolume(fakes.OS_VOLUME_1),
            fakes.OSVolume(fakes.OS_VOLUME_2),
            fakes.OSVolume(fakes.OS_VOLUME_3)]
        self.nova_admin.servers.list.return_value = [
            fakes.OSInstance_full(fakes.OS_INSTANCE_1),
            fakes.OSInstance_full(fakes.OS_INSTANCE_2)]

        self.set_mock_db_items(fakes.DB_VOLUME_1, fakes.DB_VOLUME_2,
                               fakes.DB_INSTANCE_1, fakes.DB_INSTANCE_2,
                               fakes.DB_SNAPSHOT_1, fakes.DB_SNAPSHOT_2)
        self.db_api.add_item.side_effect = (
            tools.get_db_api_add_item(fakes.ID_EC2_VOLUME_3))

        resp = self.execute('DescribeVolumes', {})
        self.assertThat(resp, matchers.DictMatches(
            {'volumeSet': [fakes.EC2_VOLUME_1, fakes.EC2_VOLUME_2,
                           fakes.EC2_VOLUME_3]},
            orderless_lists=True))

        self.db_api.get_items.assert_any_call(mock.ANY, 'vol')

        self.db_api.get_items_by_ids = tools.CopyingMock(
            return_value=[fakes.DB_VOLUME_1])
        resp = self.execute('DescribeVolumes',
                            {'VolumeId.1': fakes.ID_EC2_VOLUME_1})
        self.assertThat(resp, matchers.DictMatches(
            {'volumeSet': [fakes.EC2_VOLUME_1]},
            orderless_lists=True))
        self.db_api.get_items_by_ids.assert_any_call(
            mock.ANY, set([fakes.ID_EC2_VOLUME_1]))

        self.check_filtering(
            'DescribeVolumes', 'volumeSet',
            [('availability-zone', fakes.NAME_AVAILABILITY_ZONE),
             ('create-time', fakes.TIME_CREATE_VOLUME_2),
             ('encrypted', False),
             # TODO(ft): declare a constant for the volume size in fakes
             ('size', 1),
             ('snapshot-id', fakes.ID_EC2_SNAPSHOT_1),
             ('status', 'available'),
             ('volume-id', fakes.ID_EC2_VOLUME_1),
             # TODO(ft): support filtering by none/empty value
             # ('volume-type', ''),
             ('attachment.delete-on-termination', False),
             ('attachment.device', fakes.ROOT_DEVICE_NAME_INSTANCE_2),
             ('attachment.instance-id', fakes.ID_EC2_INSTANCE_2),
             ('attachment.status', 'attached')])
        self.check_tag_support(
            'DescribeVolumes', 'volumeSet',
            fakes.ID_EC2_VOLUME_1, 'volumeId')

    def test_describe_volumes_auto_remove(self):
        self.cinder.volumes.list.return_value = []
        self.nova.servers.list.return_value = []
        self.set_mock_db_items(fakes.DB_VOLUME_1, fakes.DB_VOLUME_2)
        resp = self.execute('DescribeVolumes', {})
        self.assertThat(resp, matchers.DictMatches(
            {'volumeSet': []}))

        self.db_api.delete_item.assert_any_call(
            mock.ANY, fakes.ID_EC2_VOLUME_1)
        self.db_api.delete_item.assert_any_call(
            mock.ANY, fakes.ID_EC2_VOLUME_2)

    def test_describe_volumes_invalid_parameters(self):
        self.cinder.volumes.list.return_value = [
            fakes.OSVolume(fakes.OS_VOLUME_1),
            fakes.OSVolume(fakes.OS_VOLUME_2)]
        self.nova.servers.list.return_value = [
            fakes.OSInstance_full(fakes.OS_INSTANCE_2)]

        self.assert_execution_error(
            'InvalidVolume.NotFound', 'DescribeVolumes',
            {'VolumeId.1': fakes.random_ec2_id('vol')})

        self.cinder.volumes.list.side_effect = lambda: []

        self.assert_execution_error(
            'InvalidVolume.NotFound', 'DescribeVolumes',
            {'VolumeId.1': fakes.ID_EC2_VOLUME_1})

    def test_create_volume(self):
        self.cinder.volumes.create.return_value = (
            fakes.OSVolume(fakes.OS_VOLUME_1))
        self.db_api.add_item.side_effect = (
            tools.get_db_api_add_item(fakes.ID_EC2_VOLUME_1))

        resp = self.execute(
            'CreateVolume',
            {'AvailabilityZone': fakes.NAME_AVAILABILITY_ZONE})
        self.assertThat(fakes.EC2_VOLUME_1, matchers.DictMatches(resp))
        self.db_api.add_item.assert_called_once_with(
            mock.ANY, 'vol',
            tools.purge_dict(fakes.DB_VOLUME_1, ('id',)))

        self.cinder.volumes.create.assert_called_once_with(
            None, snapshot_id=None, volume_type=None,
            availability_zone=fakes.NAME_AVAILABILITY_ZONE)

    def test_create_volume_from_snapshot(self):
        self.cinder.volumes.create.return_value = (
            fakes.OSVolume(fakes.OS_VOLUME_3))
        self.db_api.add_item.side_effect = (
            tools.get_db_api_add_item(fakes.ID_EC2_VOLUME_3))
        self.set_mock_db_items(fakes.DB_SNAPSHOT_1)

        resp = self.execute(
            'CreateVolume',
            {'AvailabilityZone': fakes.NAME_AVAILABILITY_ZONE,
             'SnapshotId': fakes.ID_EC2_SNAPSHOT_1})
        self.assertThat(fakes.EC2_VOLUME_3, matchers.DictMatches(resp))
        self.db_api.add_item.assert_called_once_with(
            mock.ANY, 'vol',
            tools.purge_dict(fakes.DB_VOLUME_3, ('id',)))

        self.cinder.volumes.create.assert_called_once_with(
            None, snapshot_id=fakes.ID_OS_SNAPSHOT_1, volume_type=None,
            availability_zone=fakes.NAME_AVAILABILITY_ZONE)

    def test_delete_volume(self):
        self.set_mock_db_items(fakes.DB_VOLUME_1)
        resp = self.execute('DeleteVolume',
                            {'VolumeId': fakes.ID_EC2_VOLUME_1})
        self.assertEqual({'return': True}, resp)
        self.cinder.volumes.delete.assert_called_once_with(
            fakes.ID_OS_VOLUME_1)
        self.assertFalse(self.db_api.delete_item.called)

    def test_format_volume_maps_status(self):
        fake_volume = fakes.OSVolume(fakes.OS_VOLUME_1)
        self.cinder.volumes.list.return_value = [fake_volume]
        self.nova.servers.list.return_value = []
        self.set_mock_db_items(fakes.DB_VOLUME_1)

        fake_volume.status = 'creating'
        resp = self.execute('DescribeVolumes', {})
        self.assertEqual('creating', resp['volumeSet'][0]['status'])

        fake_volume.status = 'attaching'
        resp = self.execute('DescribeVolumes', {})
        self.assertEqual('in-use', resp['volumeSet'][0]['status'])

        fake_volume.status = 'detaching'
        resp = self.execute('DescribeVolumes', {})
        self.assertEqual('in-use', resp['volumeSet'][0]['status'])

        fake_volume.status = 'banana'
        resp = self.execute('DescribeVolumes', {})
        self.assertEqual('banana', resp['volumeSet'][0]['status'])

    def test_attach_volume(self):
        self.set_mock_db_items(fakes.DB_INSTANCE_2, fakes.DB_VOLUME_3)
        os_volume = fakes.OSVolume(fakes.OS_VOLUME_3)
        os_volume.attachments.append({'device': '/dev/vdf',
                                      'server_id': fakes.ID_OS_INSTANCE_2})
        os_volume.status = 'attaching'
        self.cinder.volumes.get.return_value = os_volume

        resp = self.execute('AttachVolume',
                            {'VolumeId': fakes.ID_EC2_VOLUME_3,
                             'InstanceId': fakes.ID_EC2_INSTANCE_2,
                             'Device': '/dev/vdf'})
        self.assertEqual({'deleteOnTermination': False,
                          'device': '/dev/vdf',
                          'instanceId': fakes.ID_EC2_INSTANCE_2,
                          'status': 'attaching',
                          'volumeId': fakes.ID_EC2_VOLUME_3},
                         resp)
        self.nova.volumes.create_server_volume.assert_called_once_with(
            fakes.ID_OS_INSTANCE_2, fakes.ID_OS_VOLUME_3, '/dev/vdf')

    @mock.patch.object(fakes.OSVolume, 'get', autospec=True)
    def test_detach_volume(self, os_volume_get):
        self.set_mock_db_items(fakes.DB_INSTANCE_1, fakes.DB_INSTANCE_2,
                               fakes.DB_VOLUME_2)
        os_volume = fakes.OSVolume(fakes.OS_VOLUME_2)
        self.cinder.volumes.get.return_value = os_volume
        os_volume_get.side_effect = (
            lambda vol: setattr(vol, 'status', 'detaching'))

        resp = self.execute('DetachVolume',
                            {'VolumeId': fakes.ID_EC2_VOLUME_2})
        self.assertEqual({'device': os_volume.attachments[0]['device'],
                          'instanceId': fakes.ID_EC2_INSTANCE_2,
                          'status': 'detaching',
                          'volumeId': fakes.ID_EC2_VOLUME_2},
                         resp)
        self.nova.volumes.delete_server_volume.assert_called_once_with(
            fakes.ID_OS_INSTANCE_2, fakes.ID_OS_VOLUME_2)
        self.cinder.volumes.get.assert_called_once_with(fakes.ID_OS_VOLUME_2)

    def test_detach_volume_invalid_parameters(self):
        self.set_mock_db_items(fakes.DB_VOLUME_1)
        self.cinder.volumes.get.return_value = (
            fakes.OSVolume(fakes.OS_VOLUME_1))

        self.assert_execution_error('IncorrectState', 'DetachVolume',
                                    {'VolumeId': fakes.ID_EC2_VOLUME_1})



# Copyright 2014
# The Cloudscaling Group, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import base64
import itertools

from novaclient import exceptions as nova_exception
from oslo_cache import core as cache_core
from oslo_config import cfg
from oslo_log import log as logging
import six

from ec2api.api import clients
from ec2api.api import ec2utils
from ec2api.api import instance as instance_api
from ec2api import exception


CONF = cfg.CONF
LOG = logging.getLogger(__name__)

VERSIONS = [
    '1.0',
    '2007-01-19',
    '2007-03-01',
    '2007-08-29',
    '2007-10-10',
    '2007-12-15',
    '2008-02-01',
    '2008-09-01',
    '2009-04-04',
]

VERSION_DATA = {
    '1.0': ['ami-id',
            'ami-launch-index',
            'ami-manifest-path',
            'hostname',
            'instance-id',
            'local-ipv4',
            'public-keys',
            'reservation-id',
            'security-groups'],
    '2007-01-19': ['local-hostname',
                   'public-hostname',
                   'public-ipv4'],
    '2007-03-01': ['product-codes'],
    '2007-08-29': ['instance-type'],
    '2007-10-10': ['ancestor-ami-ids',
                   'ramdisk-id'],
    '2007-12-15': ['block-device-mapping'],
    '2008-02-01': ['kernel-id',
                   'placement'],
    '2008-09-01': ['instance-action'],
    '2009-04-04': [],
}


def get_version_list():
    return _format_metadata_item(VERSIONS + ["latest"])


def get_os_instance_and_project_id_by_provider_id(context, provider_id,
                                                  fixed_ip):
    neutron = clients.neutron(context)
    os_subnets = neutron.list_subnets(advanced_service_providers=[provider_id],
                                      fields=['network_id'])
    if not os_subnets:
        raise exception.EC2MetadataNotFound()
    os_networks = [subnet['network_id']
                   for subnet in os_subnets['subnets']]
    try:
        os_port = neutron.list_ports(
            fixed_ips='ip_address=' + fixed_ip,
            network_id=os_networks,
            fields=['device_id', 'tenant_id'])['ports'][0]
    except IndexError:
        raise exception.EC2MetadataNotFound()
    os_instance_id = os_port['device_id']
    project_id = os_port['tenant_id']
    return os_instance_id, project_id


def get_metadata_item(context, path_tokens, os_instance_id, remote_ip,
                      cache_region):
    version = path_tokens[0]
    if version == "latest":
        version = VERSIONS[-1]
    elif version not in VERSIONS:
        raise exception.EC2MetadataNotFound()

    cache_key = 'ec2api-metadata-%s' % os_instance_id
    cache = cache_region.get(
        cache_key, expiration_time=CONF.metadata.cache_expiration)
    if cache and cache != cache_core.NO_VALUE:
        _check_instance_owner(context, os_instance_id, cache['owner_id'])
        LOG.debug("Using cached metadata for instance %s", os_instance_id)
    else:
        ec2_instance, ec2_reservation = (
            _get_ec2_instance_and_reservation(context, os_instance_id))

        _check_instance_owner(context, os_instance_id,
                              ec2_reservation['ownerId'])

        metadata = _build_metadata(context, ec2_instance, ec2_reservation,
                                   os_instance_id, remote_ip)
        cache = {'metadata': metadata,
                 'owner_id': ec2_reservation['ownerId']}

        cache_region.set(cache_key, cache)

    metadata = cache['metadata']
    metadata = _cut_down_to_version(metadata, version)
    metadata_item = _find_path_in_tree(metadata, path_tokens[1:])
    return _format_metadata_item(metadata_item)


def _get_ec2_instance_and_reservation(context, os_instance_id):
    instance_id = ec2utils.os_id_to_ec2_id(context, 'i', os_instance_id)
    try:
        ec2_reservations = instance_api.describe_instances(
                context, [instance_id])
    except exception.InvalidInstanceIDNotFound:
        ec2_reservations = instance_api.describe_instances(
                context, filter=[{'name': 'instance-id',
                                  'value': [instance_id]}])
    if (len(ec2_reservations['reservationSet']) != 1 or
            len(ec2_reservations['reservationSet'][0]['instancesSet']) != 1):
        LOG.error('Failed to get metadata for instance id: %s',
                  os_instance_id)
        raise exception.EC2MetadataNotFound()

    ec2_reservation = ec2_reservations['reservationSet'][0]
    ec2_instance = ec2_reservation['instancesSet'][0]

    return ec2_instance, ec2_reservation


def _check_instance_owner(context, os_instance_id, owner_id):
    # NOTE(ft): check for case of Neutron metadata proxy.
    # It sends project_id as X-Tenant-ID HTTP header.
    # We make sure it's correct
    if context.project_id != owner_id:
        LOG.warning('Tenant_id %(tenant_id)s does not match tenant_id '
                    'of instance %(instance_id)s.',
                    {'tenant_id': context.project_id,
                     'instance_id': os_instance_id})
        raise exception.EC2MetadataNotFound()


def _build_metadata(context, ec2_instance, ec2_reservation,
                    os_instance_id, remote_ip):
    metadata = {
        'ami-id': ec2_instance['imageId'],
        'ami-launch-index': ec2_instance['amiLaunchIndex'],
        # NOTE (ft): the fake value as it is in Nova EC2 metadata
        'ami-manifest-path': 'FIXME',
        # NOTE (ft): empty value as it is in Nova EC2 metadata
        'ancestor-ami-ids': [],
        'block-device-mapping': _build_block_device_mappings(context,
                                                             ec2_instance,
                                                             os_instance_id),
        # NOTE(ft): Nova EC2 metadata returns instance's hostname with
        # dhcp_domain suffix if it's set in config.
        # But i don't see any reason to return a hostname differs from EC2
        # describe output one. If we need to consider dhcp_domain suffix
        # then we should do it in the describe operation
        'hostname': ec2_instance['privateDnsName'],
        # NOTE (ft): the fake value as it is in Nova EC2 metadata
        'instance-action': 'none',
        'instance-id': ec2_instance['instanceId'],
        'instance-type': ec2_instance['instanceType'],
        'local-hostname': ec2_instance['privateDnsName'],
        'local-ipv4': ec2_instance['privateIpAddress'] or remote_ip,
        'placement': {
            'availability-zone': ec2_instance['placement']['availabilityZone']
        },
        # NOTE (ft): empty value as it is in Nova EC2 metadata
        'product-codes': [],
        'public-hostname': ec2_instance['dnsName'],
        'public-ipv4': ec2_instance.get('ipAddress', ''),
        'reservation-id': ec2_reservation['reservationId'],
        'security-groups': [sg['groupName']
                            for sg in ec2_reservation.get('groupSet', [])],
    }
    if 'kernelId' in ec2_instance:
        metadata['kernel-id'] = ec2_instance['kernelId']
    if 'ramdiskId' in ec2_instance:
        metadata['ramdisk-id'] = ec2_instance['ramdiskId']
    # public keys are strangely rendered in ec2 metadata service
    #  meta-data/public-keys/ returns '0=keyname' (with no trailing /)
    # and only if there is a public key given.
    # '0=keyname' means there is a normally rendered dict at
    #  meta-data/public-keys/0
    #
    # meta-data/public-keys/ : '0=%s' % keyname
    # meta-data/public-keys/0/ : 'openssh-key'
    # meta-data/public-keys/0/openssh-key : '%s' % publickey
    if ec2_instance['keyName']:
        metadata['public-keys'] = {
            '0': {'_name': "0=" + ec2_instance['keyName']}}
        nova = clients.nova(context)
        os_instance = nova.servers.get(os_instance_id)
        try:
            keypair = nova.keypairs._get(
                '/%s/%s?user_id=%s' % (nova.keypairs.keypair_prefix,
                                       ec2_instance['keyName'],
                                       os_instance.user_id),
                'keypair')
        except nova_exception.NotFound:
            pass
        else:
            metadata['public-keys']['0']['openssh-key'] = keypair.public_key

    full_metadata = {'meta-data': metadata}

    userdata = instance_api.describe_instance_attribute(
                    context, ec2_instance['instanceId'], 'userData')
    if 'userData' in userdata:
        userdata = userdata['userData']['value']
        userdata = base64.b64decode(userdata)
        try:
            userdata = userdata.decode("utf-8")
        except UnicodeDecodeError:
            pass
        full_metadata['user-data'] = userdata

    return full_metadata


def _build_block_device_mappings(context, ec2_instance, os_instance_id):
    mappings = {'root': ec2_instance.get('rootDeviceName', ''),
                'ami': ec2utils.block_device_strip_dev(
                            ec2_instance.get('rootDeviceName', ''))}
    if 'blockDeviceMapping' in ec2_instance:
        # NOTE(yamahata): I'm not sure how ebs device should be numbered.
        #                 Right now sort by device name for deterministic
        #                 result.
        ebs_devices = [ebs['deviceName']
                       for ebs in ec2_instance['blockDeviceMapping']]
        ebs_devices.sort()
        ebs_devices = {'ebs%d' % num: ebs
                       for num, ebs in enumerate(ebs_devices)}
        mappings.update(ebs_devices)

    # TODO(ft): extend Nova API to get ephemerals and swap
    return mappings


def _cut_down_to_version(metadata, version):
    version_number = VERSIONS.index(version) + 1
    if version_number == len(VERSIONS):
        return metadata
    return {attr: metadata[attr]
            for attr in itertools.chain(
                *(VERSION_DATA[ver] for ver in VERSIONS[:version_number]))
            if attr in metadata}


def _format_metadata_item(data):
    if isinstance(data, dict):
        output = ''
        for key in sorted(data.keys()):
            if key == '_name':
                continue
            if isinstance(data[key], dict):
                if '_name' in data[key]:
                    output += str(data[key]['_name'])
                else:
                    output += key + '/'
            else:
                output += key

            output += '\n'
        return output[:-1]
    elif isinstance(data, list):
        return '\n'.join(data)
    else:
        return six.text_type(data)


def _find_path_in_tree(data, path_tokens):
    # given a dict/list tree, and a path in that tree, return data found there.
    for i in range(0, len(path_tokens)):
        if isinstance(data, dict) or isinstance(data, list):
            if path_tokens[i] in data:
                data = data[path_tokens[i]]
            else:
                raise exception.EC2MetadataNotFound()
        else:
            if i != len(path_tokens) - 1:
                raise exception.EC2MetadataNotFound()
            data = data[path_tokens[i]]
    return data



# Copyright 2014
# The Cloudscaling Group, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_config import cfg
from oslo_db import options
from oslo_log import log

from ec2api import paths
from ec2api import version


CONF = cfg.CONF

_DEFAULT_SQL_CONNECTION = 'sqlite:///' + paths.state_path_def('ec2api.sqlite')

_DEFAULT_LOG_LEVELS = ['amqp=WARN', 'amqplib=WARN', 'boto=WARN',
                       'sqlalchemy=WARN', 'suds=INFO',
                       'iso8601=WARN',
                       'requests.packages.urllib3.connectionpool=WARN',
                       'urllib3.connectionpool=WARN', 'websocket=WARN',
                       'keystonemiddleware=WARN', 'routes.middleware=WARN',
                       'stevedore=WARN', 'keystoneclient.auth=WARN']

_DEFAULT_LOGGING_CONTEXT_FORMAT = ('%(asctime)s.%(msecs)03d %(process)d '
                                   '%(levelname)s %(name)s [%(request_id)s '
                                   '%(user_identity)s] %(instance)s'
                                   '%(message)s')


def parse_args(argv, default_config_files=None):
    log.set_defaults(_DEFAULT_LOGGING_CONTEXT_FORMAT, _DEFAULT_LOG_LEVELS)
    log.register_options(CONF)
    options.set_defaults(CONF, connection=_DEFAULT_SQL_CONNECTION)

    cfg.CONF(argv[1:],
             project='ec2api',
             version=version.version_info.version_string(),
             default_config_files=default_config_files)



# Copyright 2014
# The Cloudscaling Group, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""RequestContext: context for requests that persist through all of ec2."""

from oslo_config import cfg
from oslo_context import context
from oslo_log import log as logging
from oslo_utils import timeutils
import six

from ec2api import clients
from ec2api import exception


CONF = cfg.CONF
LOG = logging.getLogger(__name__)


class RequestContext(context.RequestContext):
    """Security context and request information.

    Represents the user taking a given action within the system.

    """

    def __init__(self, user_id, project_id, request_id=None,
                 is_admin=None, remote_address=None,
                 auth_token=None, user_name=None, project_name=None,
                 overwrite=True, service_catalog=None, api_version=None,
                 is_os_admin=None, **kwargs):
        """Parameters

            :param overwrite: Set to False to ensure that the greenthread local
                copy of the index is not overwritten.


            :param kwargs: Extra arguments that might be present, but we ignore
                because they possibly came in from older rpc messages.
        """
        user = kwargs.pop('user', None)
        tenant = kwargs.pop('tenant', None)
        super(RequestContext, self).__init__(
            auth_token=auth_token,
            user=user_id or user,
            tenant=project_id or tenant,
            is_admin=is_admin,
            request_id=request_id,
            resource_uuid=kwargs.pop('resource_uuid', None),
            overwrite=overwrite)
        # oslo_context's RequestContext.to_dict() generates this field, we can
        # safely ignore this as we don't use it.
        kwargs.pop('user_identity', None)
        self.session = kwargs.pop('session', None)
        if kwargs:
            LOG.warning('Arguments dropped when creating context: %s',
                        str(kwargs))

        self.user_id = user_id
        self.project_id = project_id
        self.remote_address = remote_address
        timestamp = timeutils.utcnow()
        if isinstance(timestamp, six.string_types):
            timestamp = timeutils.parse_strtime(timestamp)
        self.timestamp = timestamp

        self.service_catalog = service_catalog
        if self.service_catalog is None:
            # if list is empty or none
            self.service_catalog = []

        self.user_name = user_name
        self.project_name = project_name
        self.is_admin = is_admin
        # TODO(ft): call policy.check_is_admin if is_admin is None
        self.is_os_admin = is_os_admin
        self.api_version = api_version

    def to_dict(self):
        values = super(RequestContext, self).to_dict()
        # FIXME(dims): defensive hasattr() checks need to be
        # removed once we figure out why we are seeing stack
        # traces
        values.update({
            'user_id': getattr(self, 'user_id', None),
            'project_id': getattr(self, 'project_id', None),
            'is_admin': getattr(self, 'is_admin', None),
            'remote_address': getattr(self, 'remote_address', None),
            'timestamp': self.timestamp.strftime(
                timeutils.PERFECT_TIME_FORMAT) if hasattr(
                self, 'timestamp') else None,
            'request_id': getattr(self, 'request_id', None),
            'quota_class': getattr(self, 'quota_class', None),
            'user_name': getattr(self, 'user_name', None),
            'service_catalog': getattr(self, 'service_catalog', None),
            'project_name': getattr(self, 'project_name', None),
            'is_os_admin': getattr(self, 'is_os_admin', None),
            'api_version': getattr(self, 'api_version', None),
        })
        return values

    @classmethod
    def from_dict(cls, values):
        return cls(**values)


def is_user_context(context):
    """Indicates if the request context is a normal user."""
    if not context:
        return False
    if context.is_os_admin:
        return False
    if not context.user_id or not context.project_id:
        return False
    return True


def require_context(ctxt):
    """Raise exception.AuthFailure()

    if context is not a user or an admin context.
    """
    if not ctxt.is_os_admin and not is_user_context(ctxt):
        raise exception.AuthFailure()


def get_os_admin_context():
    """Create a context to interact with OpenStack as an administrator."""
    admin_session = clients.get_os_admin_session()
    return RequestContext(
        None, None,
        session=admin_session,
        is_os_admin=True,
        overwrite=False)



# Copyright 2010 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
"""
Unittets for S3 objectstore clone.
"""
from botocore import exceptions as botocore_exception
import botocore.session
import fixtures
from oslo_config import cfg
from oslo_config import fixture as config_fixture
from oslotest import base as test_base

from ec2api.s3 import s3server

CONF = cfg.CONF


class S3APITestCase(test_base.BaseTestCase):
    """Test objectstore through S3 API."""

    def setUp(self):
        """Setup users, projects, and start a test server."""
        super(S3APITestCase, self).setUp()
        tempdir = self.useFixture(fixtures.TempDir())
        conf = self.useFixture(config_fixture.Config())
        conf.config(buckets_path=tempdir.path,
                    s3_listen='127.0.0.1',
                    s3_listen_port=0)

        self.server = s3server.get_wsgi_server()
        # NOTE(ft): this requires eventlet.monkey_patch, which is called in
        # tests/unit/__init__.py. Remove it out from there if you get these
        # tests rid of server run
        self.server.start()
        self.addCleanup(self.server.stop)

        s3_url = 'http://' + CONF.s3_listen + ':' + str(self.server.port)
        region = 'FakeRegion'
        connection_data = {
            'config_file': (None, 'AWS_CONFIG_FILE', None, None),
            'region': ('region', 'BOTO_DEFAULT_REGION', region, None),
        }
        session = botocore.session.get_session(connection_data)
        conn = session.create_client(
            's3', region_name=region, endpoint_url=s3_url,
            aws_access_key_id='fake', aws_secret_access_key='fake')
        self.conn = conn

        def get_http_connection(*args):
            """Get a new S3 connection, don't attempt to reuse connections."""
            return self.conn.new_http_connection(*args)

        self.conn.get_http_connection = get_http_connection

    def _ensure_no_buckets(self, buckets):
        self.assertEqual(len(buckets['Buckets']), 0,
                         "Bucket list was not empty")
        return True

    def _ensure_one_bucket(self, buckets, name):
        self.assertEqual(len(buckets['Buckets']), 1,
                         "Bucket list didn't have exactly one element in it")
        self.assertEqual(buckets['Buckets'][0]['Name'], name, "Wrong name")
        return True

    def test_list_buckets(self):
        # Make sure we started with no buckets.
        self._ensure_no_buckets(self.conn.list_buckets())

    def test_create_and_delete_bucket(self):
        # Test bucket creation and deletion.
        bucket_name = 'testbucket'

        self.conn.create_bucket(Bucket=bucket_name)
        self._ensure_one_bucket(self.conn.list_buckets(), bucket_name)
        self.conn.delete_bucket(Bucket=bucket_name)
        self._ensure_no_buckets(self.conn.list_buckets())

    def test_create_bucket_and_key_and_delete_key(self):
        # Test key operations on buckets.
        bucket_name = 'testbucket'
        key_name = 'somekey'
        key_contents = b'somekey'

        self.conn.create_bucket(Bucket=bucket_name)
        self.conn.put_object(Bucket=bucket_name, Key=key_name,
                             Body=key_contents)

        # make sure the contents are correct
        key = self.conn.get_object(Bucket=bucket_name, Key=key_name)
        self.assertEqual(key['Body'].read(), key_contents,
                         "Bad contents")

        # delete the key
        self.conn.delete_object(Bucket=bucket_name, Key=key_name)

        self.assertRaises(botocore_exception.ClientError, self.conn.get_object,
                          Bucket=bucket_name, Key=key_name)

    def test_unknown_bucket(self):
        bucket_name = 'falalala'
        self.assertRaises(botocore_exception.ClientError,
                          self.conn.head_bucket,
                          Bucket=bucket_name)
        self.assertRaises(botocore_exception.ClientError,
                          self.conn.list_objects,
                          Bucket=bucket_name, MaxKeys=0)



# Copyright 2014
# The Cloudscaling Group, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import importlib
from unittest import mock

import fixtures

from ec2api.api import clients
from ec2api.tests.unit import base


class ClientsTestCase(base.BaseTestCase):

    def setUp(self):
        importlib.reload(clients)
        super(ClientsTestCase, self).setUp()

    @mock.patch.object(clients, '_get_nova_api_version', return_value='2.3')
    @mock.patch('novaclient.client.Client')
    def test_nova(self, nova, get_api_version):
        context = mock.NonCallableMock(session=mock.sentinel.session)

        # test normal flow with get_api_version call
        res = clients.nova(context)
        self.assertEqual(nova.return_value, res)
        nova.assert_called_with('2.3', service_type='compute',
                                session=mock.sentinel.session)
        get_api_version.assert_called_once_with(context)

        # test CONF.nova_service_type is used
        self.configure(nova_service_type='compute_legacy')
        clients.nova(context)
        nova.assert_called_with('2.3', service_type='compute_legacy',
                                session=mock.sentinel.session)

    @mock.patch('novaclient.client.Client')
    def test_get_api_version(self, nova):
        context = mock.NonCallableMock(session=mock.sentinel.session)
        v2 = mock.NonCallableMock()
        v2.configure_mock(id='v2',
                          version='',
                          links=[{'href': 'http://host:port/path/v2/'}])
        v2_1 = mock.NonCallableMock()
        v2_1.configure_mock(id='v2.1',
                            version='2.40',
                            links=[{'href': 'http://host:port/path/v2.1/'}])

        # test normal flow
        nova.return_value.versions.get_current.return_value = v2_1
        with fixtures.LoggerFixture(
                format='[%(levelname)s] %(message)s') as logs:
            res = clients._get_nova_api_version(context)
        self.assertEqual(clients.REQUIRED_NOVA_API_MICROVERSION, res)
        nova.assert_called_with('2.1', service_type='compute',
                                session=mock.sentinel.session)
        nova.return_value.versions.get_current.assert_called_with()
        self.assertTrue(logs.output.startswith('[INFO]'))

        # test Nova doesn't supprt required microversion
        v2_1.version = '2.2'
        with fixtures.LoggerFixture(
                format='[%(levelname)s] %(message)s') as logs:
            res = clients._get_nova_api_version(context)
        self.assertEqual('2.2', res)
        self.assertTrue(logs.output.startswith('[WARNING]'))

        # test service type is not v2.1
        nova.return_value.versions.get_current.return_value = v2
        self.configure(nova_service_type='compute_legacy')
        with fixtures.LoggerFixture(
                format='[%(levelname)s] %(message)s') as logs:
            res = clients._get_nova_api_version(context)
        self.assertEqual('2', res)
        self.assertTrue(logs.output.startswith('[WARNING]'))
        self.configure(nova_service_type='compute')

        # test service url is not found in version list
        nova.return_value.versions.get_current.return_value = None
        with fixtures.LoggerFixture(
                format='[%(levelname)s] %(message)s') as logs:
            res = clients._get_nova_api_version(context)
        self.assertEqual(clients.REQUIRED_NOVA_API_MICROVERSION, res)
        self.assertTrue(logs.output.startswith('[WARNING]'))

    @mock.patch('neutronclient.v2_0.client.Client')
    def test_neutron(self, neutron):
        context = mock.NonCallableMock(session=mock.sentinel.session)
        res = clients.neutron(context)
        self.assertEqual(neutron.return_value, res)
        neutron.assert_called_with(service_type='network',
                                   session=mock.sentinel.session)

    @mock.patch('glanceclient.client.Client')
    def test_glance(self, glance):
        context = mock.NonCallableMock(session=mock.sentinel.session)
        res = clients.glance(context)
        self.assertEqual(glance.return_value, res)
        glance.assert_called_with(version='2', service_type='image',
                                  session=mock.sentinel.session)

    @mock.patch('cinderclient.client.Client')
    def test_cinder(self, cinder):
        # test normal flow
        context = mock.NonCallableMock(session=mock.sentinel.session)
        res = clients.cinder(context)
        self.assertEqual(cinder.return_value, res)
        cinder.assert_called_with('2', service_type='volumev3',
                                  session=mock.sentinel.session)

    @mock.patch('keystoneclient.client.Client')
    def test_keystone(self, keystone):
        context = mock.NonCallableMock(session=mock.sentinel.session)
        res = clients.keystone(context)
        self.assertEqual(keystone.return_value, res)
        keystone.assert_called_with(auth_url='v2',
                                    session=mock.sentinel.session)



# Copyright 2014
# The Cloudscaling Group, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from ec2api.tests.unit import tools

XML_RESULT_TEMPLATE = '''
<%(action)sResponse
    xmlns="http://ec2.amazonaws.com/doc/%(api_version)s/">
  <requestId>%(request_id)s</requestId>
  %(data)s
</%(action)sResponse>'''
XML_ERROR_TEMPLATE = '''
<Response>
    <Errors><Error>
        <Code>%(code)s</Code><Message>%(message)s</Message>
    </Error></Errors>
    <RequestID>%(request_id)s</RequestID>
</Response>'''

XML_FAKE_RESULT = '''<FakeActionResponse
xmlns="http://ec2.amazonaws.com/doc/2010-08-31/">
  <fakeInfo>
    <fakeNone/>
    <fakeTrue>true</fakeTrue>
    <fakeFalse>false</fakeFalse>
    <fakeInt>1234</fakeInt>
    <fakeStr>fake</fakeStr>
    <fakeSet>
      <item>
        <fakeData>fake</fakeData>
      </item>
      <item>
        <fakeData>fake</fakeData>
      </item>
    </fakeSet>
  </fakeInfo>
  <fakeEmptySet/>
  <fakeComplexSet>
    <item>
      <fakeSubSet>
        <item>
          <fakeData>fake</fakeData>
        </item>
        <item>
          <fakeData/>
        </item>
      </fakeSubSet>
    </item>
    <item>
      <fakeSubSet>
        <item>
          <fakeData>fake</fakeData>
        </item>
        <item>
          <fakeData>fake</fakeData>
        </item>
      </fakeSubSet>
    </item>
  </fakeComplexSet>
  <requestId/>
</FakeActionResponse>
'''
DICT_FAKE_RESULT_DATA = {
    'fakeInfo': {
        'fakeNone': None,
        'fakeTrue': True,
        'fakeFalse': False,
        'fakeInt': 1234,
        'fakeStr': 'fake',
        'fakeSet': [{'fakeData': 'fake'},
                    {'fakeData': 'fake'}],
    },
    'fakeEmptySet': [],
    'fakeComplexSet': [
        {'fakeSubSet': [{'fakeData': 'fake'},
                        {'fakeData': None}]},
        {'fakeSubSet': [{'fakeData': 'fake'},
                        {'fakeData': 'fake'}]},
    ],
}
DICT_FAKE_RESULT = {
    'FakeActionResponse': tools.update_dict(
        DICT_FAKE_RESULT_DATA,
        {'requestId': None})
}

XML_SINGLE_RESULT = '''
<CreateSnapshotResponse xmlns="http://ec2.amazonaws.com/doc/2009-11-30/">
  <requestId>req-8a80bb71-1e1d-49be-819f-fba429b0ddf1</requestId>
  <status>pending</status>
  <description/>
  <volumeId>vol-00000001</volumeId>
  <volumeSize>1</volumeSize>
  <progress/>
  <startTime>2014-06-04T19:55:55.448117</startTime>
  <ownerId/>
  <snapshotId>snap-00000001</snapshotId>
</CreateSnapshotResponse>
'''
DICT_SINGLE_RESULT = {
    'CreateSnapshotResponse': {
        'status': 'pending',
        'description': None,
        'volumeId': 'vol-00000001',
        'volumeSize': 1,
        'progress': None,
        'startTime': '2014-06-04T19:55:55.448117',
        'ownerId': None,
        'snapshotId': 'snap-00000001',
        'requestId': 'req-8a80bb71-1e1d-49be-819f-fba429b0ddf1',
    }
}

XML_RESULT_SET = '''
<DescribeImagesResponse xmlns="http://ec2.amazonaws.com/doc/2010-08-31/">
  <requestId>req-1fc541a8-477d-4928-a90e-4448ea57ba51</requestId>
  <imagesSet>
    <item>
      <description/>
      <imageOwnerId>77dcabaee8ea4a8fbae697ddc09afdaf</imageOwnerId>
      <isPublic>true</isPublic>
      <imageId>aki-00000001</imageId>
      <imageState>available</imageState>
      <architecture/>
      <imageLocation>None (cirros-0.3.2-x86_64-uec-kernel)</imageLocation>
      <rootDeviceType>instance-store</rootDeviceType>
      <rootDeviceName>/dev/sda1</rootDeviceName>
      <imageType>kernel</imageType>
      <name>cirros-0.3.2-x86_64-uec-kernel</name>
    </item>
    <item>
      <description/>
      <imageOwnerId>77dcabaee8ea4a8fbae697ddc09afdaf</imageOwnerId>
      <isPublic>true</isPublic>
      <imageId>ari-00000002</imageId>
      <imageState>available</imageState>
      <architecture/>
      <imageLocation>None (cirros-0.3.2-x86_64-uec-ramdisk)</imageLocation>
      <rootDeviceType>instance-store</rootDeviceType>
      <rootDeviceName>/dev/sda1</rootDeviceName>
      <imageType>ramdisk</imageType>
      <name>cirros-0.3.2-x86_64-uec-ramdisk</name>
    </item>
    <item>
      <name>cirros-0.3.2-x86_64-uec</name>
      <imageOwnerId>77dcabaee8ea4a8fbae697ddc09afdaf</imageOwnerId>
      <isPublic>true</isPublic>
      <imageId>ami-00000003</imageId>
      <imageState>available</imageState>
      <rootDeviceType>instance-store</rootDeviceType>
      <architecture/>
      <imageLocation>None (cirros-0.3.2-x86_64-uec)</imageLocation>
      <kernelId>aki-00000001</kernelId>
      <ramdiskId>ari-00000002</ramdiskId>
      <rootDeviceName>/dev/sda1</rootDeviceName>
      <imageType>machine</imageType>
      <description/>
    </item>
    <item>
      <description/>
      <imageOwnerId>77dcabaee8ea4a8fbae697ddc09afdaf</imageOwnerId>
      <isPublic>true</isPublic>
      <imageId>ami-00000004</imageId>
      <imageState>available</imageState>
      <architecture/>
      <imageLocation>None (Fedora-x86_64-20-20131211.1-sda)</imageLocation>
      <rootDeviceType>instance-store</rootDeviceType>
      <rootDeviceName>/dev/sda1</rootDeviceName>
      <imageType>machine</imageType>
      <name>Fedora-x86_64-20-20131211.1-sda</name>
    </item>
  </imagesSet>
</DescribeImagesResponse>
'''
DICT_RESULT_SET = {
    'DescribeImagesResponse': {
        'imagesSet': [{
            'description': None,
            'imageOwnerId': '77dcabaee8ea4a8fbae697ddc09afdaf',
            'isPublic': True,
            'imageId': 'aki-00000001',
            'imageState': 'available',
            'architecture': None,
            'imageLocation': 'None (cirros-0.3.2-x86_64-uec-kernel)',
            'rootDeviceType': 'instance-store',
            'rootDeviceName': '/dev/sda1',
            'imageType': 'kernel',
            'name': 'cirros-0.3.2-x86_64-uec-kernel',
        },
            {
            'description': None,
            'imageOwnerId': '77dcabaee8ea4a8fbae697ddc09afdaf',
            'isPublic': True,
            'imageId': 'ari-00000002',
            'imageState': 'available',
            'architecture': None,
            'imageLocation': 'None (cirros-0.3.2-x86_64-uec-ramdisk)',
            'rootDeviceType': 'instance-store',
            'rootDeviceName': '/dev/sda1',
            'imageType': 'ramdisk',
            'name': 'cirros-0.3.2-x86_64-uec-ramdisk',
        },
            {
            'name': 'cirros-0.3.2-x86_64-uec',
            'imageOwnerId': '77dcabaee8ea4a8fbae697ddc09afdaf',
            'isPublic': True,
            'imageId': 'ami-00000003',
            'imageState': 'available',
            'rootDeviceType': 'instance-store',
            'architecture': None,
            'imageLocation': 'None (cirros-0.3.2-x86_64-uec)',
            'kernelId': 'aki-00000001',
            'ramdiskId': 'ari-00000002',
            'rootDeviceName': '/dev/sda1',
            'imageType': 'machine',
            'description': None,
        },
            {
            'description': None,
            'imageOwnerId': '77dcabaee8ea4a8fbae697ddc09afdaf',
            'isPublic': True,
            'imageId': 'ami-00000004',
            'imageState': 'available',
            'architecture': None,
            'imageLocation': 'None (Fedora-x86_64-20-20131211.1-sda)',
            'rootDeviceType': 'instance-store',
            'rootDeviceName': '/dev/sda1',
            'imageType': 'machine',
            'name': 'Fedora-x86_64-20-20131211.1-sda',
        }],
        'requestId': 'req-1fc541a8-477d-4928-a90e-4448ea57ba51',
    }
}

XML_EMPTY_RESULT_SET = '''<?xml version="1.0" encoding="UTF-8"?>
<DescribeVolumesResponse xmlns="http://ec2.amazonaws.com/doc/2014-05-01/">
    <requestId>a25fa489-f97f-428a-9d30-9fcb1e9b9b65</requestId>
    <volumeSet/>
</DescribeVolumesResponse>
'''
DICT_EMPTY_RESULT_SET = {
    'DescribeVolumesResponse': {
        'requestId': 'a25fa489-f97f-428a-9d30-9fcb1e9b9b65',
        'volumeSet': [],
    }
}

XML_ERROR = '''<?xml version="1.0"?>
<Response><Errors><Error><Code>InvalidInstanceID.NotFound</Code>
<Message>Instance i-00000001 could not be found.</Message></Error></Errors>
<RequestID>req-89eb083f-3c44-46e7-bc37-2c050ed7a9ce</RequestID></Response>
'''
DICT_ERROR = {
    'Response': {
        'RequestID': 'req-89eb083f-3c44-46e7-bc37-2c050ed7a9ce',
        'Errors': {
            'Error': {
                'Code': 'InvalidInstanceID.NotFound',
                'Message': 'Instance i-00000001 could not be found.',
            }
        }
    }
}

XML_SILENT_OPERATIN_RESULT = '''
<DeleteVpcResponse xmlns="http://ec2.amazonaws.com/doc/2009-11-30/">
  <requestId>req-8a80bb71-1e1d-49be-819f-fba429b0ddf1</requestId>
  <return>true</return>
</DeleteVpcResponse>
'''

DOTTED_FAKE_PARAMS = {
    'FakeStr': 'fake',
    'FakeInt': '1234',
    'FakeBool': 'False',
    'FakeDict.FakeKey': 'fake',
    'FakeList.1.FakeElemKey': 'fake',
    'FakeList.2.FakeElemKey': 'fake',
    'FakeComplexList.1.FakeElemKey.1.FakeSubElemKey': 'fake',
    'FakeComplexList.1.FakeElemKey.2.FakeSubElemKey': 'fake',
    'FakeComplexList.1.FakeElemKeyOther': 'fake',
    'FakeComplexList.2.FakeElemKey.1.FakeSubElemKey': 'fake',
    'FakeComplexList.2.FakeElemKey.2.FakeSubElemKey': 'fake',
    'FakeComplexList.2.FakeElemKeyOther': 'fake',
}
DICT_FAKE_PARAMS = {
    'fake_str': 'fake',
    'fake_int': 1234,
    'fake_bool': False,
    'fake_dict': {'fake_key': 'fake'},
    'fake_list': [{'fake_elem_key': 'fake'},
                  {'fake_elem_key': 'fake'}],
    'fake_complex_list': [
        {'fake_elem_key': [{'fake_sub_elem_key': 'fake'},
                           {'fake_sub_elem_key': 'fake'}],
         'fake_elem_key_other': 'fake'},
        {'fake_elem_key': [{'fake_sub_elem_key': 'fake'},
                           {'fake_sub_elem_key': 'fake'}],
         'fake_elem_key_other': 'fake'}],
}



# Copyright 2010 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# Copyright 2010 OpenStack Foundation
# Copyright 2009 Facebook
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""Implementation of an S3-like storage server based on local files.

Useful to test features that will eventually run on S3, or if you want to
run something locally that was once running on S3.

We don't support all the features of S3, but it does work with the
standard S3 client for the most basic semantics. To use the standard
S3 client with this module::

    c = S3.AWSAuthConnection("", "", server="localhost", port=8888,
                             is_secure=False)
    c.create_bucket("mybucket")
    c.put("mybucket", "mykey", "a value")
    print c.get("mybucket", "mykey").body

"""

import bisect
import datetime
import os.path

from oslo_config import cfg
from oslo_log import log as logging
from oslo_utils import fileutils
import routes
import six
from six.moves.urllib import parse
import webob

from ec2api import paths
from ec2api import utils
from ec2api import wsgi


s3_opts = [
    cfg.StrOpt('buckets_path',
               default=paths.state_path_def('buckets'),
               help='Path to S3 buckets'),
    cfg.StrOpt('s3_listen',
               default="0.0.0.0",
               help='IP address for S3 API to listen'),
    cfg.IntOpt('s3_listen_port',
               default=3334,
               help='Port for S3 API to listen'),
]

CONF = cfg.CONF
CONF.register_opts(s3_opts)
LOG = logging.getLogger(__name__)


def get_wsgi_server():
    return wsgi.Server("S3 Objectstore",
                       S3Application(CONF.buckets_path),
                       port=CONF.s3_listen_port,
                       host=CONF.s3_listen)


class S3Application(wsgi.Router):
    """Implementation of an S3-like storage server based on local files.

    If bucket depth is given, we break files up into multiple directories
    to prevent hitting file system limits for number of files in each
    directories. 1 means one level of directories, 2 means 2, etc.

    """

    def __init__(self, root_directory, bucket_depth=0, mapper=None):
        if mapper is None:
            mapper = routes.Mapper()

        mapper.connect(
                '/',
                controller=lambda *a, **kw: RootHandler(self)(*a, **kw))
        mapper.connect(
                '/{bucket}/{object_name}',
                controller=lambda *a, **kw: ObjectHandler(self)(*a, **kw))
        mapper.connect(
                '/{bucket_name}',
                controller=lambda *a, **kw: BucketHandler(self)(*a, **kw),
                requirements={'bucket_name': '[^/]+/?'})
        self.directory = os.path.abspath(root_directory)
        fileutils.ensure_tree(self.directory)
        self.bucket_depth = bucket_depth
        super(S3Application, self).__init__(mapper)


class BaseRequestHandler(object):
    """Base class emulating Tornado's web framework pattern in WSGI.

    This is a direct port of Tornado's implementation, so some key decisions
    about how the code interacts have already been chosen.

    The two most common ways of designing web frameworks can be
    classified as async object-oriented and sync functional.

    Tornado's is on the OO side because a response is built up in and using
    the shared state of an object and one of the object's methods will
    eventually trigger the "finishing" of the response asynchronously.

    Most WSGI stuff is in the functional side, we pass a request object to
    every call down a chain and the eventual return value will be a response.

    Part of the function of the routing code in S3Application as well as the
    code in BaseRequestHandler's __call__ method is to merge those two styles
    together enough that the Tornado code can work without extensive
    modifications.

    To do that it needs to give the Tornado-style code clean objects that it
    can modify the state of for each request that is processed, so we use a
    very simple factory lambda to create new state for each request, that's
    the stuff in the router, and when we let the Tornado code modify that
    object to handle the request, then we return the response it generated.
    This wouldn't work the same if Tornado was being more async'y and doing
    other callbacks throughout the process, but since Tornado is being
    relatively simple here we can be satisfied that the response will be
    complete by the end of the get/post method.

    """

    def __init__(self, application):
        self.application = application

    @webob.dec.wsgify
    def __call__(self, request):
        try:
            method = request.method.lower()
            f = getattr(self, method, self.invalid)
            self.request = request
            self.response = webob.Response()
            params = request.environ['wsgiorg.routing_args'][1]
            del params['controller']
            f(**params)
        except Exception:
            # TODO(andrey-mp): improve this block
            LOG.exception('Unhandled error')
            self.render_xml({"Error": {
                "Code": "BadRequest",
                "Message": "Unhandled error"
            }})
            self.set_status(501)

        return self.response

    def get_argument(self, arg, default):
        return self.request.params.get(arg, default)

    def set_header(self, header, value):
        self.response.headers[header] = value

    def set_status(self, status_code):
        self.response.status = status_code

    def set_404(self):
        self.render_xml({"Error": {
            "Code": "NoSuchKey",
            "Message": "The resource you requested does not exist"
        }})
        self.set_status(404)

    def finish(self, body=''):
        if isinstance(body, six.binary_type):
            self.response.body = body
        else:
            self.response.body = body.encode("utf-8")

    def invalid(self, **kwargs):
        pass

    def render_xml(self, value):
        assert isinstance(value, dict) and len(value) == 1
        self.set_header("Content-Type", "application/xml; charset=UTF-8")
        name = next(six.iterkeys(value))
        parts = []
        parts.append('<' + name +
                     ' xmlns="http://s3.amazonaws.com/doc/2006-03-01/">')
        self._render_parts(next(six.itervalues(value)), parts)
        parts.append('</' + name + '>')
        self.finish('<?xml version="1.0" encoding="UTF-8"?>\n' +
                    ''.join(parts))

    def _render_parts(self, value, parts=None):
        if not parts:
            parts = []

        if isinstance(value, six.string_types):
            parts.append(utils.xhtml_escape(value))
        elif isinstance(value, six.integer_types):
            parts.append(str(value))
        elif isinstance(value, datetime.datetime):
            parts.append(value.strftime("%Y-%m-%dT%H:%M:%S.000Z"))
        elif isinstance(value, dict):
            for name, subvalue in value.items():
                if not isinstance(subvalue, list):
                    subvalue = [subvalue]
                for subsubvalue in subvalue:
                    parts.append('<' + name + '>')
                    self._render_parts(subsubvalue, parts)
                    parts.append('</' + name + '>')
        else:
            raise Exception("Unknown S3 value type %r", value)

    def _object_path(self, bucket, object_name):
        if self.application.bucket_depth < 1:
            return os.path.abspath(os.path.join(
                self.application.directory, bucket, object_name))
        name_hash = utils.get_hash_str(object_name)
        path = os.path.abspath(os.path.join(
            self.application.directory, bucket))
        for i in range(self.application.bucket_depth):
            path = os.path.join(path, name_hash[:2 * (i + 1)])
        return os.path.join(path, object_name)


class RootHandler(BaseRequestHandler):
    def get(self):
        names = os.listdir(self.application.directory)
        buckets = []
        for name in names:
            path = os.path.join(self.application.directory, name)
            info = os.stat(path)
            buckets.append({
                "Name": name,
                "CreationDate": datetime.datetime.utcfromtimestamp(
                    info.st_ctime),
            })
        self.render_xml({"ListAllMyBucketsResult": {
            "Buckets": {"Bucket": buckets},
        }})


class BucketHandler(BaseRequestHandler):
    def get(self, bucket_name):
        prefix = self.get_argument("prefix", u"")
        marker = self.get_argument("marker", u"")
        max_keys = int(self.get_argument("max-keys", 50000))
        path = os.path.abspath(os.path.join(self.application.directory,
                                            bucket_name))
        terse = int(self.get_argument("terse", 0))
        if (not path.startswith(self.application.directory) or
                not os.path.isdir(path)):
            self.set_404()
            return
        object_names = []
        for root, _dirs, files in os.walk(path):
            for file_name in files:
                object_names.append(os.path.join(root, file_name))
        skip = len(path) + 1
        for i in range(self.application.bucket_depth):
            skip += 2 * (i + 1) + 1
        object_names = [n[skip:] for n in object_names]
        object_names.sort()
        contents = []

        start_pos = 0
        if marker:
            start_pos = bisect.bisect_right(object_names, marker, start_pos)
        if prefix:
            start_pos = bisect.bisect_left(object_names, prefix, start_pos)

        truncated = False
        for object_name in object_names[start_pos:]:
            if not object_name.startswith(prefix):
                break
            if len(contents) >= max_keys:
                truncated = True
                break
            object_path = self._object_path(bucket_name, object_name)
            c = {"Key": object_name}
            if not terse:
                info = os.stat(object_path)
                c.update({
                    "LastModified": datetime.datetime.utcfromtimestamp(
                        info.st_mtime),
                    "Size": info.st_size,
                })
            contents.append(c)
            marker = object_name
        self.render_xml({"ListBucketResult": {
            "Name": bucket_name,
            "Prefix": prefix,
            "Marker": marker,
            "MaxKeys": max_keys,
            "IsTruncated": truncated,
            "Contents": contents,
        }})

    def put(self, bucket_name):
        path = os.path.abspath(os.path.join(
            self.application.directory, bucket_name))
        if (not path.startswith(self.application.directory) or
                os.path.exists(path)):
            self.set_status(403)
            return
        fileutils.ensure_tree(path)
        self.finish()

    def delete(self, bucket_name):
        path = os.path.abspath(os.path.join(
            self.application.directory, bucket_name))
        if (not path.startswith(self.application.directory) or
                not os.path.isdir(path)):
            self.set_404()
            return
        if len(os.listdir(path)) > 0:
            self.set_status(403)
            return
        os.rmdir(path)
        self.set_status(204)
        self.finish()

    def head(self, bucket_name):
        path = os.path.abspath(os.path.join(self.application.directory,
                                            bucket_name))
        if (not path.startswith(self.application.directory) or
                not os.path.isdir(path)):
            self.set_404()
            return
        self.set_status(200)
        self.finish()


class ObjectHandler(BaseRequestHandler):
    def get(self, bucket, object_name):
        object_name = parse.unquote(object_name)
        path = self._object_path(bucket, object_name)
        if (not path.startswith(self.application.directory) or
                not os.path.isfile(path)):
            self.set_404()
            return
        info = os.stat(path)
        self.set_header("Content-Type", "application/unknown")
        self.set_header("Last-Modified", datetime.datetime.utcfromtimestamp(
            info.st_mtime))
        object_file = open(path, "rb")
        try:
            self.finish(object_file.read())
        finally:
            object_file.close()

    def put(self, bucket, object_name):
        object_name = parse.unquote(object_name)
        bucket_dir = os.path.abspath(os.path.join(
            self.application.directory, bucket))
        if (not bucket_dir.startswith(self.application.directory) or
                not os.path.isdir(bucket_dir)):
            self.set_404()
            return
        path = self._object_path(bucket, object_name)
        if not path.startswith(bucket_dir) or os.path.isdir(path):
            self.set_status(403)
            return
        directory = os.path.dirname(path)
        fileutils.ensure_tree(directory)
        object_file = open(path, "wb")
        object_file.write(self.request.body)
        object_file.close()
        self.set_header('ETag',
                        '"%s"' % utils.get_hash_str(self.request.body))
        self.finish()

    def delete(self, bucket, object_name):
        object_name = parse.unquote(object_name)
        path = self._object_path(bucket, object_name)
        if (not path.startswith(self.application.directory) or
                not os.path.isfile(path)):
            self.set_404()
            return
        os.unlink(path)
        self.set_status(204)
        self.finish()



# Copyright 2014
# The Cloudscaling Group, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Starting point for routing EC2 requests.
"""
import hashlib
import sys

import botocore
from keystoneauth1 import session as keystone_session
from keystoneclient import access as keystone_access
from keystoneclient.auth.identity import access as keystone_identity_access
from oslo_config import cfg
from oslo_context import context as common_context
from oslo_log import log as logging
from oslo_serialization import jsonutils
from oslo_utils import timeutils
import requests
import six
import webob
import webob.dec
import webob.exc

from ec2api.api import apirequest
from ec2api.api import ec2utils
from ec2api.api import faults
from ec2api import clients
from ec2api import context
from ec2api import exception
from ec2api.i18n import _
from ec2api import wsgi


LOG = logging.getLogger(__name__)

ec2_opts = [
    cfg.StrOpt('keystone_ec2_tokens_url',
               default='http://localhost:5000/v3/ec2tokens',
               help='URL to authenticate token from ec2 request.'),
    cfg.IntOpt('ec2_timestamp_expiry',
               default=300,
               help='Time in seconds before ec2 timestamp expires'),
]

CONF = cfg.CONF
CONF.register_opts(ec2_opts)
CONF.import_opt('use_forwarded_for', 'ec2api.api.auth')


# Fault Wrapper around all EC2 requests #
class FaultWrapper(wsgi.Middleware):

    """Calls the middleware stack, captures any exceptions into faults."""

    @webob.dec.wsgify(RequestClass=wsgi.Request)
    def __call__(self, req):
        try:
            return req.get_response(self.application)
        except Exception:
            LOG.exception("FaultWrapper catches error")
            return faults.Fault(webob.exc.HTTPInternalServerError())


class RequestLogging(wsgi.Middleware):

    """Access-Log akin logging for all EC2 API requests."""

    @webob.dec.wsgify(RequestClass=wsgi.Request)
    def __call__(self, req):
        start = timeutils.utcnow()
        rv = req.get_response(self.application)
        self.log_request_completion(rv, req, start)
        return rv

    def log_request_completion(self, response, request, start):
        apireq = request.environ.get('ec2.request', None)
        if apireq:
            action = apireq.action
        else:
            action = None
        ctxt = request.environ.get('ec2api.context', None)
        delta = timeutils.utcnow() - start
        seconds = delta.seconds
        microseconds = delta.microseconds
        LOG.info(
            "%s.%ss %s %s %s %s %s [%s] %s %s",
            seconds,
            microseconds,
            request.remote_addr,
            request.method,
            "%s%s" % (request.script_name, request.path_info),
            action,
            response.status_int,
            request.user_agent,
            request.content_type,
            response.content_type,
            context=ctxt)


class EC2KeystoneAuth(wsgi.Middleware):

    """Authenticate an EC2 request with keystone and convert to context."""

    def _get_signature(self, req):
        """Extract the signature from the request.

        This can be a get/post variable or for version 4 also in a header
        called 'Authorization'.
        - params['Signature'] == version 0,1,2,3
        - params['X-Amz-Signature'] == version 4
        - header 'Authorization' == version 4
        """
        sig = req.params.get('Signature') or req.params.get('X-Amz-Signature')
        if sig is not None:
            return sig

        if 'Authorization' not in req.headers:
            return None

        auth_str = req.headers['Authorization']
        if not auth_str.startswith('AWS4-HMAC-SHA256'):
            return None

        return auth_str.partition("Signature=")[2].split(',')[0]

    def _get_access(self, req):
        """Extract the access key identifier.

        For version 0/1/2/3 this is passed as the AccessKeyId parameter, for
        version 4 it is either an X-Amz-Credential parameter or a Credential=
        field in the 'Authorization' header string.
        """
        access = req.params.get('AWSAccessKeyId')
        if access is not None:
            return access

        cred_param = req.params.get('X-Amz-Credential')
        if cred_param:
            access = cred_param.split("/")[0]
            if access is not None:
                return access

        if 'Authorization' not in req.headers:
            return None
        auth_str = req.headers['Authorization']
        if not auth_str.startswith('AWS4-HMAC-SHA256'):
            return None
        cred_str = auth_str.partition("Credential=")[2].split(',')[0]
        return cred_str.split("/")[0]

    @webob.dec.wsgify(RequestClass=wsgi.Request)
    def __call__(self, req):
        request_id = common_context.generate_request_id()

        # NOTE(alevine) We need to calculate the hash here because
        # subsequent access to request modifies the req.body so the hash
        # calculation will yield invalid results.
        body_hash = hashlib.sha256(req.body).hexdigest()

        signature = self._get_signature(req)
        if not signature:
            msg = _("Signature not provided")
            return faults.ec2_error_response(request_id, "AuthFailure", msg,
                                             status=400)
        access = self._get_access(req)
        if not access:
            msg = _("Access key not provided")
            return faults.ec2_error_response(request_id, "AuthFailure", msg,
                                             status=400)

        if 'X-Amz-Signature' in req.params or 'Authorization' in req.headers:
            params = {}
        else:
            # Make a copy of args for authentication and signature verification
            params = dict(req.params)
            # Not part of authentication args
            params.pop('Signature', None)

        cred_dict = {
            'access': access,
            'signature': signature,
            'host': req.host,
            'verb': req.method,
            'path': req.path,
            'params': params,
            # python3 takes only keys for json from headers object
            'headers': {k: req.headers[k] for k in req.headers},
            'body_hash': body_hash
        }

        token_url = CONF.keystone_ec2_tokens_url
        if "ec2" in token_url:
            creds = {'ec2Credentials': cred_dict}
        else:
            creds = {'auth': {'OS-KSEC2:ec2Credentials': cred_dict}}
        creds_json = jsonutils.dumps(creds)
        headers = {'Content-Type': 'application/json'}
        params = {'data': creds_json, 'headers': headers}
        clients.update_request_params_with_ssl(params)
        response = requests.request('POST', token_url, **params)
        status_code = response.status_code
        if status_code != 200:
            msg = response.reason
            return faults.ec2_error_response(request_id, "AuthFailure", msg,
                                             status=status_code)

        try:
            auth_ref = keystone_access.AccessInfo.factory(resp=response,
                                                          body=response.json())
        except (NotImplementedError, KeyError):
            LOG.exception("Keystone failure")
            msg = _("Failure communicating with keystone")
            return faults.ec2_error_response(request_id, "AuthFailure", msg,
                                             status=400)
        auth = keystone_identity_access.AccessInfoPlugin(auth_ref)
        params = {'auth': auth}
        clients.update_request_params_with_ssl(params)
        session = keystone_session.Session(**params)
        remote_address = req.remote_addr
        if CONF.use_forwarded_for:
            remote_address = req.headers.get('X-Forwarded-For',
                                             remote_address)

        ctxt = context.RequestContext(auth_ref.user_id, auth_ref.project_id,
                                      request_id=request_id,
                                      user_name=auth_ref.username,
                                      project_name=auth_ref.project_name,
                                      remote_address=remote_address,
                                      session=session,
                                      api_version=req.params.get('Version'))

        req.environ['ec2api.context'] = ctxt

        return self.application


class Requestify(wsgi.Middleware):

    @webob.dec.wsgify(RequestClass=wsgi.Request)
    def __call__(self, req):
        non_args = ['Action', 'Signature', 'AWSAccessKeyId', 'SignatureMethod',
                    'SignatureVersion', 'Version', 'Timestamp']
        args = dict(req.params)
        try:
            expired = ec2utils.is_ec2_timestamp_expired(
                req.params,
                expires=CONF.ec2_timestamp_expiry)
            if expired:
                msg = _("Timestamp failed validation.")
                LOG.exception(msg)
                raise webob.exc.HTTPForbidden(explanation=msg)

            # Raise KeyError if omitted
            action = req.params['Action']
            # Fix bug lp:720157 for older (version 1) clients
            version = req.params.get('SignatureVersion')
            if version and int(version) == 1:
                non_args.remove('SignatureMethod')
                if 'SignatureMethod' in args:
                    args.pop('SignatureMethod')
            for non_arg in non_args:
                args.pop(non_arg, None)
        except KeyError:
            raise webob.exc.HTTPBadRequest()
        except exception.InvalidRequest as err:
            raise webob.exc.HTTPBadRequest(explanation=err.format_message())

        LOG.debug('action: %s', action)
        for key, value in args.items():
            LOG.debug('arg: %(key)s\t\tval: %(value)s',
                      {'key': key, 'value': value})

        # Success!
        api_request = apirequest.APIRequest(
            action, req.params['Version'], args)
        req.environ['ec2.request'] = api_request
        return self.application


def exception_to_ec2code(ex):
    """Helper to extract EC2 error code from exception.

    For other than EC2 exceptions (those without ec2_code attribute),
    use exception name.
    """
    if hasattr(ex, 'ec2_code'):
        code = ex.ec2_code
    else:
        code = type(ex).__name__
    return code


def ec2_error_ex(ex, req, unexpected=False):
    """Return an EC2 error response.

    Return an EC2 error response based on passed exception and log
    the exception on an appropriate log level:

        * DEBUG: expected errors
        * ERROR: unexpected errors

    All expected errors are treated as client errors and 4xx HTTP
    status codes are always returned for them.

    Unexpected 5xx errors may contain sensitive information,
    suppress their messages for security.
    """
    code = exception_to_ec2code(ex)
    for status_name in ('code', 'status', 'status_code', 'http_status'):
        status = getattr(ex, status_name, None)
        if isinstance(status, int):
            break
    else:
        status = 500

    if unexpected:
        log_fun = LOG.error
        log_msg = _("Unexpected %(ex_name)s raised: %(ex_str)s")
        exc_info = sys.exc_info()
    else:
        log_fun = LOG.debug
        log_msg = _("%(ex_name)s raised: %(ex_str)s")
        exc_info = None

    context = req.environ['ec2api.context']
    request_id = context.request_id
    log_msg_args = {
        'ex_name': type(ex).__name__,
        'ex_str': ex
    }
    log_fun(log_msg % log_msg_args, context=context, exc_info=exc_info)

    if unexpected and status >= 500:
        message = _('Unknown error occurred.')
    elif getattr(ex, 'message', None):
        message = six.text_type(ex.message)
    elif ex.args and any(arg for arg in ex.args):
        message = " ".join(map(six.text_type, ex.args))
    else:
        message = six.text_type(ex)
    if unexpected:
        # Log filtered environment for unexpected errors.
        env = req.environ.copy()
        for k in list(env.keys()):
            if not isinstance(env[k], six.string_types):
                env.pop(k)
        log_fun(_('Environment: %s') % jsonutils.dumps(env))
    return faults.ec2_error_response(request_id, code, message, status=status)


class Executor(wsgi.Application):

    """Execute an EC2 API request.

    Executes 'ec2.action', passing 'ec2api.context' and
    'ec2.action_args' (all variables in WSGI environ.)  Returns an XML
    response, or a 400 upon failure.
    """

    @webob.dec.wsgify(RequestClass=wsgi.Request)
    def __call__(self, req):
        context = req.environ['ec2api.context']
        api_request = req.environ['ec2.request']
        try:
            result = api_request.invoke(context)
        except botocore.exceptions.ClientError as ex:
            error = ex.response.get('Error', {})
            code = ex.response.get('Code', error.get('Code'))
            message = ex.response.get('Message', error.get('Message'))
            # the early versions of botocore didn't provide HTTPStatusCode
            # for 400 errors
            status = ex.response.get('ResponseMetadata', {}).get(
                'HTTPStatusCode', 400)
            if status < 400 or status > 499:
                LOG.exception("Exception from remote server")
            return faults.ec2_error_response(
                context.request_id, code, message, status=status)
        except Exception as ex:
            return ec2_error_ex(
                ex, req, unexpected=not isinstance(ex, exception.EC2Exception))
        else:
            resp = webob.Response()
            resp.status = 200
            resp.headers['Content-Type'] = 'text/xml'
            resp.body = six.binary_type(result)

            return resp



# Copyright 2014
# The Cloudscaling Group, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import netaddr
from oslo_config import cfg
from oslo_log import log as logging

from ec2api.api import common
from ec2api.api import ec2utils
from ec2api import clients
from ec2api.db import api as db_api
from ec2api import exception
from ec2api.i18n import _


LOG = logging.getLogger(__name__)

ec2_opts = [
    cfg.IntOpt('network_device_mtu',
               default=1500,
               help='MTU size to set by DHCP for instances. Corresponds '
                    'with the network_device_mtu in ec2api.conf.')
]

CONF = cfg.CONF
CONF.register_opts(ec2_opts)


"""DHCP options related API implementation
"""


class Validator(common.Validator):

    def dopt_id_or_default(self, id):
        if id == 'default':
            return
        self.ec2_id(id, ['dopt'])


DHCP_OPTIONS_MAP = {'domain-name-servers': 'dns-server',
                    'domain-name': 'domain-name',
                    'ntp-servers': 'ntp-server',
                    'netbios-name-servers': 'netbios-ns',
                    'netbios-node-type': 'netbios-nodetype'}


def create_dhcp_options(context, dhcp_configuration):
    dhcp_options = {}
    for dhcp_option in dhcp_configuration:
        key = dhcp_option['key']
        values = dhcp_option['value']
        if key not in DHCP_OPTIONS_MAP:
            raise exception.InvalidParameterValue(
                        value=values,
                        parameter=key,
                        reason='Unrecognized key is specified')
        if not type(values) is list:
            raise exception.InvalidParameterValue(
                value=values,
                parameter=key,
                reason='List of values is expected')
        if key not in ['domain-name', 'netbios-node-type']:
            ips = []
            for ip in values:
                ip_address = netaddr.IPAddress(ip)
                if not ip_address:
                    raise exception.InvalidParameterValue(
                        value=ip,
                        parameter=key,
                        reason='Invalid list of IPs is specified')
                ips.append(ip)
            dhcp_options[key] = ips
        else:
            dhcp_options[key] = values
    dhcp_options = db_api.add_item(context, 'dopt',
                                   {'dhcp_configuration': dhcp_options})
    return {'dhcpOptions': _format_dhcp_options(context, dhcp_options)}


def delete_dhcp_options(context, dhcp_options_id):
    if not dhcp_options_id:
        raise exception.MissingParameter(
            _('DHCP options ID must be specified'))
    dhcp_options = ec2utils.get_db_item(context, dhcp_options_id)
    vpcs = db_api.get_items(context, 'vpc')
    for vpc in vpcs:
        if dhcp_options['id'] == vpc.get('dhcp_options_id'):
            raise exception.DependencyViolation(
                        obj1_id=dhcp_options['id'],
                        obj2_id=vpc['id'])
    db_api.delete_item(context, dhcp_options['id'])
    return True


class DhcpOptionsDescriber(common.TaggableItemsDescriber,
                           common.NonOpenstackItemsDescriber):

    KIND = 'dopt'
    FILTER_MAP = {'dhcp_options_id': 'dhcpOptionsId',
                  'key': ['dhcpConfigurationSet', 'key'],
                  'value': ['dhcpConfigurationSet', ['valueSet', 'value']]}

    def format(self, dhcp_options):
        return _format_dhcp_options(self.context, dhcp_options)


def describe_dhcp_options(context, dhcp_options_id=None,
                          filter=None):
    formatted_dhcp_options = DhcpOptionsDescriber().describe(
            context, ids=dhcp_options_id, filter=filter)
    return {'dhcpOptionsSet': formatted_dhcp_options}


def associate_dhcp_options(context, dhcp_options_id, vpc_id):
    vpc = ec2utils.get_db_item(context, vpc_id)
    rollback_dhcp_options_id = vpc.get('dhcp_options_id')
    if dhcp_options_id == 'default':
        dhcp_options_id = None
        dhcp_options = None
    else:
        dhcp_options = ec2utils.get_db_item(context, dhcp_options_id)
        dhcp_options_id = dhcp_options['id']
    neutron = clients.neutron(context)
    os_ports = neutron.list_ports(tenant_id=context.project_id)['ports']
    network_interfaces = db_api.get_items(context, 'eni')
    rollback_dhcp_options_object = (
            db_api.get_item_by_id(context, rollback_dhcp_options_id)
            if dhcp_options_id is not None else
            None)
    with common.OnCrashCleaner() as cleaner:
        _associate_vpc_item(context, vpc, dhcp_options_id)
        cleaner.addCleanup(_associate_vpc_item, context, vpc,
                           rollback_dhcp_options_id)
        for network_interface in network_interfaces:
            os_port = next((p for p in os_ports
                            if p['id'] == network_interface['os_id']), None)
            if not os_port:
                continue
            _add_dhcp_opts_to_port(context, dhcp_options,
                                   network_interface, os_port, neutron)
            cleaner.addCleanup(_add_dhcp_opts_to_port, context,
                               rollback_dhcp_options_object, network_interface,
                               os_port, neutron)
    return True


def _add_dhcp_opts_to_port(context, dhcp_options, network_interface, os_port,
                           neutron=None):
    dhcp_opts = [{'opt_name': 'mtu',
                  'opt_value': str(CONF.network_device_mtu)}]
    if dhcp_options is not None:
        for key, values in dhcp_options['dhcp_configuration'].items():
            strvalues = [str(v) for v in values]
            dhcp_opts.append({'opt_name': DHCP_OPTIONS_MAP[key],
                              'opt_value': ','.join(strvalues)})
    if not neutron:
        neutron = clients.neutron(context)
    neutron.update_port(os_port['id'],
                        {'port': {'extra_dhcp_opts': dhcp_opts}})


def _format_dhcp_options(context, dhcp_options):
    dhcp_configuration = []
    for key, values in dhcp_options['dhcp_configuration'].items():
        items = [{'value': v} for v in values]
        dhcp_configuration.append({'key': key, 'valueSet': items})
    return {'dhcpOptionsId': dhcp_options['id'],
            'dhcpConfigurationSet': dhcp_configuration}


def _associate_vpc_item(context, vpc, dhcp_options_id):
    vpc['dhcp_options_id'] = dhcp_options_id
    db_api.update_item(context, vpc)



# Copyright 2014
# The Cloudscaling Group, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
from unittest import mock

import six
import tempfile

from cinderclient import exceptions as cinder_exception
import eventlet
from oslo_concurrency import processutils
from oslo_serialization import jsonutils

from ec2api.api import image as image_api
from ec2api import exception
from ec2api.tests.unit import base
from ec2api.tests.unit import fakes
from ec2api.tests.unit import matchers
from ec2api.tests.unit import tools

AMI_MANIFEST_XML = """<?xml version="1.0" ?>
<manifest>
        <version>2011-06-17</version>
        <bundler>
                <name>test-s3</name>
                <version>0</version>
                <release>0</release>
        </bundler>
        <machine_configuration>
                <architecture>x86_64</architecture>
                <block_device_mapping>
                        <mapping>
                                <virtual>ami</virtual>
                                <device>sda1</device>
                        </mapping>
                        <mapping>
                                <virtual>root</virtual>
                                <device>/dev/sda1</device>
                        </mapping>
                        <mapping>
                                <virtual>ephemeral0</virtual>
                                <device>sda2</device>
                        </mapping>
                        <mapping>
                                <virtual>swap</virtual>
                                <device>sda3</device>
                        </mapping>
                </block_device_mapping>
                <kernel_id>%(aki-id)s</kernel_id>
                <ramdisk_id>%(ari-id)s</ramdisk_id>
        </machine_configuration>
        <image>
                <ec2_encrypted_key>foo</ec2_encrypted_key>
                <user_encrypted_key>foo</user_encrypted_key>
                <ec2_encrypted_iv>foo</ec2_encrypted_iv>
                <parts count="1">
                        <part index="0">
                               <filename>foo</filename>
                        </part>
                </parts>
        </image>
</manifest>
""" % {'aki-id': fakes.ID_EC2_IMAGE_AKI_1,
       'ari-id': fakes.ID_EC2_IMAGE_ARI_1}

FILE_MANIFEST_XML = """<?xml version="1.0" ?>
<manifest>
        <image>
                <ec2_encrypted_key>foo</ec2_encrypted_key>
                <user_encrypted_key>foo</user_encrypted_key>
                <ec2_encrypted_iv>foo</ec2_encrypted_iv>
                <parts count="1">
                        <part index="0">
                               <filename>foo</filename>
                        </part>
                </parts>
        </image>
</manifest>
"""


class ImageTestCase(base.ApiTestCase):

    @mock.patch('ec2api.api.instance._is_ebs_instance')
    def _test_create_image(self, instance_status, no_reboot, is_ebs_instance):
        self.set_mock_db_items(fakes.DB_INSTANCE_2)
        os_instance = mock.MagicMock()
        os_instance.configure_mock(id=fakes.ID_OS_INSTANCE_2,
                                   status=instance_status)
        stop_called = iter([False, True])
        os_instance.stop.side_effect = lambda: next(stop_called)
        os_instance.get.side_effect = lambda: (setattr(os_instance, 'status',
                                                       'SHUTOFF')
                                               if next(stop_called) else None)
        image_id = fakes.random_ec2_id('ami')
        os_image_id = fakes.random_os_id()
        os_instance.create_image.return_value = os_image_id
        self.glance.images.get.return_value = fakes.OSImage(
            {'id': os_image_id},
            from_get=True)
        self.nova.servers.get.return_value = os_instance
        is_ebs_instance.return_value = True
        self.db_api.add_item.side_effect = tools.get_db_api_add_item(image_id)

        resp = self.execute('CreateImage',
                            {'InstanceId': fakes.ID_EC2_INSTANCE_2,
                             'Name': 'fake_name',
                             'Description': 'fake desc',
                             'NoReboot': str(no_reboot)})
        self.assertEqual({'imageId': image_id},
                         resp)
        self.db_api.get_item_by_id.assert_called_once_with(
            mock.ANY, fakes.ID_EC2_INSTANCE_2)
        self.nova.servers.get.assert_called_once_with(fakes.ID_OS_INSTANCE_2)
        is_ebs_instance.assert_called_once_with(mock.ANY, os_instance.id)
        expected_image = {'is_public': False,
                          'description': 'fake desc'}
        if no_reboot:
            expected_image['os_id'] = os_image_id
        self.db_api.add_item.assert_called_once_with(
            mock.ANY, 'ami', expected_image)
        if not no_reboot:
            eventlet.sleep()
        if not no_reboot:
            os_instance.stop.assert_called_once_with()
            os_instance.get.assert_called_once_with()
            os_instance.start.assert_called_once_with()
        if no_reboot:
            os_instance.create_image.assert_called_once_with('fake_name')
        else:
            os_instance.create_image.assert_called_once_with(
                'fake_name', metadata={'ec2_id': image_id})
            self.db_api.update_item.assert_called_once_with(
                mock.ANY, {'id': image_id,
                           'is_public': False,
                           'description': 'fake desc',
                           'os_id': os_image_id,
                           'vpc_id': None})

        self.db_api.reset_mock()
        self.nova.servers.reset_mock()

    def test_create_image(self):
        self._test_create_image('ACTIVE', False)
        self._test_create_image('SHUTOFF', True)

    @mock.patch('ec2api.api.instance._is_ebs_instance')
    def test_register_image_by_url(self, is_ebs_instance):
        self.set_mock_db_items(fakes.DB_INSTANCE_2)
        is_ebs_instance.return_value = True

        # Setup the mock parameters
        image_id = fakes.random_ec2_id('ami')
        os_image_id = fakes.random_os_id()
        self.glance.images.create.return_value = fakes.OSImage(
            {'id': os_image_id},
            from_get=True)
        self.db_api.add_item.side_effect = tools.get_db_api_add_item(image_id)

        # Setup Import Command
        import_command = 'RegisterImage'

        # Setup the import arguments
        args = {
            'Name': 'TestImage123',
            'ImageLocation':
                fakes.LOCATION_IMAGE_2,
            'Architecture': 'x86_64'
        }

        # Execute the import image process
        resp = self.execute(import_command, args)

        # Assert that the image returned is equal to what was expected
        self.assertEqual({'imageId': image_id}, resp)

        # Assert that Glance Image Create was called
        self.glance.images.create.assert_called_once_with(
            name='TestImage123',
            disk_format='raw',
            container_format='bare',
            visibility='private',
            architecture='x86_64',
            image_location=fakes.LOCATION_IMAGE_2)

        # Assert that Glance Image Import was called
        self.glance.images.image_import.assert_called_once_with(
            os_image_id,
            method='web-download',
            uri=fakes.LOCATION_IMAGE_2)

        # Assert that the image was created
        expected_image = {'is_public': False,
                          'os_id': mock.ANY,
                          'description': None}
        self.db_api.add_item.assert_called_once_with(
            mock.ANY, 'ami', expected_image)

        # Reset all test settings/state
        self.db_api.reset_mock()
        self.glance.reset_mock()

    @mock.patch('ec2api.api.instance._is_ebs_instance')
    def test_create_image_invalid_parameters(self, is_ebs_instance):
        self.set_mock_db_items(fakes.DB_INSTANCE_1)
        is_ebs_instance.return_value = False

        self.assert_execution_error('InvalidParameterValue', 'CreateImage',
                                    {'InstanceId': fakes.ID_EC2_INSTANCE_1,
                                     'Name': 'fake_name'})

    @mock.patch('ec2api.api.image._s3_create')
    def test_register_image_by_s3(self, s3_create):
        s3_create.return_value = fakes.OSImage(fakes.OS_IMAGE_1)
        self.db_api.add_item.side_effect = (
            tools.get_db_api_add_item(fakes.ID_EC2_IMAGE_1))

        resp = self.execute(
            'RegisterImage',
            {'ImageLocation': fakes.LOCATION_IMAGE_1})
        self.assertThat(resp, matchers.DictMatches(
            {'imageId': fakes.ID_EC2_IMAGE_1}))

        s3_create.assert_called_once_with(
            mock.ANY,
            {'name': fakes.LOCATION_IMAGE_1,
             'image_location': fakes.LOCATION_IMAGE_1})
        s3_create.reset_mock()

        resp = self.execute(
            'RegisterImage',
            {'ImageLocation': fakes.LOCATION_IMAGE_1,
             'Name': 'an image name'})
        self.assertThat(resp, matchers.DictMatches(
            {'imageId': fakes.ID_EC2_IMAGE_1}))

        s3_create.assert_called_once_with(
            mock.ANY,
            {'name': 'an image name',
             'image_location': fakes.LOCATION_IMAGE_1})

    @mock.patch('ec2api.api.ec2utils.get_os_image')
    def test_register_image_by_bdm(self, get_os_image):
        self.glance.images.create.return_value = (
            fakes.OSImage(fakes.OS_IMAGE_2))
        self.glance.images.upload.return_value = (
            fakes.OSImage(fakes.OS_IMAGE_2))
        self.cinder.volume_snapshots.get.side_effect = (
            tools.get_by_1st_arg_getter(
                {fakes.ID_OS_SNAPSHOT_1: (
                    fakes.OSSnapshot(fakes.OS_SNAPSHOT_1))},
                notfound_exception=cinder_exception.NotFound(404)))
        self.db_api.add_item.side_effect = (
            tools.get_db_api_add_item(fakes.ID_EC2_IMAGE_2))
        self.set_mock_db_items(fakes.DB_SNAPSHOT_1, fakes.DB_SNAPSHOT_2,
                               fakes.DB_IMAGE_AKI_1, fakes.DB_IMAGE_ARI_1)
        get_os_image.side_effect = [fakes.OSImage(fakes.OS_IMAGE_AKI_1),
                                    fakes.OSImage(fakes.OS_IMAGE_ARI_1)]

        resp = self.execute(
            'RegisterImage',
            {'RootDeviceName': fakes.ROOT_DEVICE_NAME_IMAGE_2,
             'Name': 'fake_name',
             'KernelId': fakes.ID_EC2_IMAGE_AKI_1,
             'RamdiskId': fakes.ID_EC2_IMAGE_ARI_1,
             'BlockDeviceMapping.1.DeviceName': fakes.ROOT_DEVICE_NAME_IMAGE_2,
             'BlockDeviceMapping.1.Ebs.SnapshotId': fakes.ID_EC2_SNAPSHOT_1,
             'BlockDeviceMapping.2.DeviceName': '/dev/vdf',
             'BlockDeviceMapping.2.Ebs.VolumeSize': '100',
             'BlockDeviceMapping.2.Ebs.DeleteOnTermination': 'False',
             'BlockDeviceMapping.3.DeviceName': '/dev/vdg',
             'BlockDeviceMapping.3.Ebs.SnapshotId': fakes.ID_EC2_SNAPSHOT_1,
             'BlockDeviceMapping.3.Ebs.VolumeSize': '55',
             'BlockDeviceMapping.3.Ebs.DeleteOnTermination': 'True',
             'BlockDeviceMapping.4.DeviceName': '/dev/vdh',
             'BlockDeviceMapping.4.Ebs.SnapshotId': fakes.ID_EC2_SNAPSHOT_2})
        self.assertThat(resp, matchers.DictMatches(
            {'imageId': fakes.ID_EC2_IMAGE_2}))
        self.db_api.add_item.assert_called_once_with(
            mock.ANY, 'ami', {'os_id': fakes.ID_OS_IMAGE_2,
                              'is_public': False,
                              'description': None})
        self.assertEqual(1, self.glance.images.create.call_count)
        self.assertEqual((), self.glance.images.create.call_args[0])
        self.assertIsInstance(
            self.glance.images.create.call_args[1], dict)
        bdm = self.glance.images.create.call_args[1].pop(
            'block_device_mapping', 'null')
        self.assertEqual(
            {'visibility': 'private',
             'name': 'fake_name',
             'kernel_id': fakes.ID_OS_IMAGE_AKI_1,
             'ramdisk_id': fakes.ID_OS_IMAGE_ARI_1,
             'root_device_name': fakes.ROOT_DEVICE_NAME_IMAGE_2,
             'container_format': 'bare',
             'disk_format': 'raw',
             'bdm_v2': 'True'},
            self.glance.images.create.call_args[1])
        self.assertEqual([{'boot_index': 0,
                           'delete_on_termination': True,
                           'destination_type': 'volume',
                           'device_name': fakes.ROOT_DEVICE_NAME_IMAGE_2,
                           'source_type': 'snapshot',
                           'snapshot_id': fakes.ID_OS_SNAPSHOT_1,
                           'volume_size': 1},
                          {'boot_index': -1,
                           'delete_on_termination': False,
                           'destination_type': 'volume',
                           'device_name': '/dev/vdf',
                           'source_type': 'blank',
                           'volume_size': 100},
                          {'boot_index': -1,
                           'delete_on_termination': True,
                           'destination_type': 'volume',
                           'device_name': '/dev/vdg',
                           'source_type': 'snapshot',
                           'snapshot_id': fakes.ID_OS_SNAPSHOT_1,
                           'volume_size': 55},
                          {'boot_index': -1,
                           'delete_on_termination': True,
                           'destination_type': 'volume',
                           'device_name': '/dev/vdh',
                           'source_type': 'snapshot',
                           'snapshot_id': fakes.ID_OS_SNAPSHOT_2}],
                         jsonutils.loads(bdm))
        get_os_image.assert_has_calls(
            [mock.call(mock.ANY, fakes.ID_EC2_IMAGE_AKI_1),
             mock.call(mock.ANY, fakes.ID_EC2_IMAGE_ARI_1)])
        self.cinder.volume_snapshots.get.assert_any_call(
            fakes.ID_OS_SNAPSHOT_1)

    def test_register_image_invalid_parameters(self):
        self.assert_execution_error(
            'InvalidParameterCombination', 'RegisterImage', {})

    def test_deregister_image(self):
        self._setup_model()

        # normal flow
        resp = self.execute('DeregisterImage',
                            {'ImageId': fakes.ID_EC2_IMAGE_1})
        self.assertThat(resp, matchers.DictMatches({'return': True}))
        self.db_api.delete_item.assert_called_once_with(
            mock.ANY, fakes.ID_EC2_IMAGE_1)
        self.glance.images.delete.assert_called_once_with(
            fakes.ID_OS_IMAGE_1)

        # deregister image which failed on asynchronously creation
        self.glance.reset_mock()
        image_id = fakes.random_ec2_id('ami')
        self.add_mock_db_items({'id': image_id,
                                'os_id': None,
                                'state': 'failed'})
        resp = self.execute('DeregisterImage',
                            {'ImageId': image_id})
        self.assertThat(resp, matchers.DictMatches({'return': True}))
        self.db_api.delete_item.assert_called_with(mock.ANY, image_id)
        self.assertFalse(self.glance.images.delete.called)

    def test_deregister_image_invalid_parameters(self):
        self._setup_model()

        self.assert_execution_error('InvalidAMIID.NotFound', 'DeregisterImage',
                                    {'ImageId': fakes.random_ec2_id('ami')})

        # deregister asynchronously creating image
        image_id = fakes.random_ec2_id('ami')
        self.add_mock_db_items({'id': image_id,
                                'os_id': None})
        self.assert_execution_error('IncorrectState',
                                    'DeregisterImage',
                                    {'ImageId': image_id})

    def test_describe_images(self):
        self._setup_model()

        resp = self.execute('DescribeImages', {})
        self.assertThat(
            resp,
            matchers.DictMatches(
                {'imagesSet': [fakes.EC2_IMAGE_1, fakes.EC2_IMAGE_2]},
                orderless_lists=True),
            verbose=True)

        self.db_api.get_items.assert_any_call(mock.ANY, 'ami')
        self.db_api.get_items.assert_any_call(mock.ANY, 'aki')
        self.db_api.get_items.assert_any_call(mock.ANY, 'ari')

        self.db_api.get_items_by_ids = tools.CopyingMock(
            side_effect=self.db_api.get_items_by_ids.side_effect)

        resp = self.execute('DescribeImages',
                            {'ImageId.1': fakes.ID_EC2_IMAGE_1})
        self.assertThat(resp,
                        matchers.DictMatches(
                            {'imagesSet': [fakes.EC2_IMAGE_1]},
                            orderless_lists=True))
        self.db_api.get_items_by_ids.assert_any_call(
            mock.ANY, set([fakes.ID_EC2_IMAGE_1]))

        self.check_filtering(
            'DescribeImages', 'imagesSet',
            [('architecture', 'x86_64'),
             ('block-device-mapping.device-name', '/dev/sdb2'),
             ('block-device-mapping.snapshot-id', fakes.ID_EC2_SNAPSHOT_1),
             ('block-device-mapping.volume-size', 22),
             ('description', 'fake desc'),
             ('image-id', fakes.ID_EC2_IMAGE_1),
             ('image-type', 'machine'),
             ('is-public', True),
             ('kernel_id', fakes.ID_EC2_IMAGE_AKI_1,),
             ('name', 'fake_name'),
             ('owner-id', fakes.ID_OS_PROJECT),
             ('ramdisk-id', fakes.ID_EC2_IMAGE_ARI_1),
             ('root-device-name', fakes.ROOT_DEVICE_NAME_IMAGE_1),
             ('root-device-type', 'instance-store'),
             ('state', 'available')])
        self.check_tag_support(
            'DescribeImages', 'imagesSet',
            fakes.ID_EC2_IMAGE_1, 'imageId',
            ('ami', 'ari', 'aki'))

    def test_describe_images_invalid_parameters(self):
        self._setup_model()

        self.assert_execution_error('InvalidAMIID.NotFound', 'DescribeImages',
                                    {'ImageId.1': fakes.random_ec2_id('ami')})

        self.glance.images.list.side_effect = lambda: []

        self.assert_execution_error('InvalidAMIID.NotFound', 'DescribeImages',
                                    {'ImageId.1': fakes.ID_EC2_IMAGE_1})

    def test_describe_image_attributes(self):
        self._setup_model()

        def do_check(attr, ec2_image_id, response):
            resp = self.execute('DescribeImageAttribute',
                                {'ImageId': ec2_image_id,
                                 'Attribute': attr})
            response['imageId'] = ec2_image_id
            self.assertThat(resp,
                            matchers.DictMatches(response,
                                                 orderless_lists=True),
                            verbose=True)

        do_check('launchPermission',
                 fakes.ID_EC2_IMAGE_2,
                 {'launchPermission': [{'group': 'all'}]})

        do_check('kernel',
                 fakes.ID_EC2_IMAGE_1,
                 {'kernel': {'value': fakes.ID_EC2_IMAGE_AKI_1}})

        do_check('ramdisk',
                 fakes.ID_EC2_IMAGE_1,
                 {'ramdisk': {'value': fakes.ID_EC2_IMAGE_ARI_1}})

        do_check('rootDeviceName',
                 fakes.ID_EC2_IMAGE_1,
                 {'rootDeviceName': fakes.ROOT_DEVICE_NAME_IMAGE_1})

        do_check('rootDeviceName',
                 fakes.ID_EC2_IMAGE_2,
                 {'rootDeviceName': fakes.ROOT_DEVICE_NAME_IMAGE_2})

        do_check('blockDeviceMapping',
                 fakes.ID_EC2_IMAGE_1,
                 {'blockDeviceMapping': (
                        fakes.EC2_IMAGE_1['blockDeviceMapping'])})

        do_check('blockDeviceMapping',
                 fakes.ID_EC2_IMAGE_2,
                 {'blockDeviceMapping': (
                        fakes.EC2_IMAGE_2['blockDeviceMapping'])})

    def test_describe_image_attributes_invalid_parameters(self):
        image_id = fakes.random_ec2_id('ami')
        self.set_mock_db_items({'id': image_id,
                                'os_id': None})
        self.assert_execution_error('IncorrectState',
                                    'DescribeImageAttribute',
                                    {'ImageId': image_id,
                                     'Attribute': 'kernel'})

    def test_modify_image_attributes(self):
        self._setup_model()

        resp = self.execute('ModifyImageAttribute',
                            {'imageId': fakes.ID_EC2_IMAGE_1,
                             'attribute': 'launchPermission',
                             'operationType': 'add',
                             'userGroup.1': 'all'})
        self.assertThat(resp, matchers.DictMatches({'return': True}))
        self.glance.images.update.assert_called_once_with(
                fakes.ID_OS_IMAGE_1, visibility='public')

    def test_modify_image_attributes_invalid_parameters(self):
        image_id = fakes.random_ec2_id('ami')
        self.set_mock_db_items({'id': image_id,
                                'os_id': None})
        self.assert_execution_error('IncorrectState',
                                    'ModifyImageAttribute',
                                    {'ImageId': image_id,
                                     'Attribute': 'kernel'})

    def _setup_model(self):
        self.set_mock_db_items(fakes.DB_IMAGE_1, fakes.DB_IMAGE_2,
                               fakes.DB_SNAPSHOT_1, fakes.DB_SNAPSHOT_2,
                               fakes.DB_IMAGE_AKI_1, fakes.DB_IMAGE_ARI_1,
                               fakes.DB_VOLUME_1, fakes. DB_VOLUME_2)
        self.db_api.get_public_items.return_value = []

        # NOTE(ft): glance.image.list returns an iterator, not just a list
        self.glance.images.list.side_effect = (
            lambda: (fakes.OSImage(i)
                     for i in (fakes.OS_IMAGE_1, fakes.OS_IMAGE_2)))
        self.glance.images.get.side_effect = (
            lambda os_id: (fakes.OSImage(fakes.OS_IMAGE_1, from_get=True)
                           if os_id == fakes.ID_OS_IMAGE_1 else
                           fakes.OSImage(fakes.OS_IMAGE_2, from_get=True)
                           if os_id == fakes.ID_OS_IMAGE_2 else
                           None))


class ImagePrivateTestCase(base.BaseTestCase):

    def test_format_image(self):
        image_ids = {fakes.ID_OS_IMAGE_1: fakes.ID_EC2_IMAGE_1,
                     fakes.ID_OS_IMAGE_AKI_1: fakes.ID_EC2_IMAGE_AKI_1,
                     fakes.ID_OS_IMAGE_ARI_1: fakes.ID_EC2_IMAGE_ARI_1}
        os_image = {'id': fakes.ID_OS_IMAGE_1,
                    'owner': fakes.ID_OS_PROJECT,
                    'created_at': fakes.TIME_CREATE_IMAGE,
                    'visibility': 'private',
                    'status': 'active',
                    'container_format': 'ami',
                    'name': 'fake_name'}

        # check name and location attributes for an unnamed image
        os_image['image_location'] = 'location'
        os_image['name'] = None

        image = image_api._format_image(
                'fake_context', fakes.DB_IMAGE_1, fakes.OSImage(os_image),
                None, image_ids)

        self.assertEqual('location', image['imageLocation'])
        self.assertEqual('location', image['name'])

        # check name and location attributes for complete image
        os_image['image_location'] = None
        os_image['name'] = 'fake_name'

        image = image_api._format_image(
                'fake_context', fakes.DB_IMAGE_1, fakes.OSImage(os_image),
                None, image_ids)

        self.assertEqual('None (fake_name)', image['imageLocation'])
        self.assertEqual('fake_name', image['name'])

        # check ebs image type for bdm_v2 mapping type
        os_image['bdm_v2'] = True
        os_image['root_device_name'] = '/dev/vda'
        os_image['block_device_mapping'] = [
            {'boot_index': 0,
             'snapshot_id': fakes.ID_OS_SNAPSHOT_2,
             'source_type': 'snapshot',
             'destination_type': 'volume'}]

        image = image_api._format_image(
                'fake_context', fakes.DB_IMAGE_1, fakes.OSImage(os_image),
                None, image_ids,
                snapshot_ids={fakes.ID_OS_SNAPSHOT_2: fakes.ID_EC2_SNAPSHOT_2})

        self.assertEqual('ebs', image['rootDeviceType'])

        # check instance-store image attributes with no any device mappings
        os_image['bdm_v2'] = False
        os_image['root_device_name'] = '/dev/vda'
        os_image['block_device_mapping'] = []
        image = image_api._format_image(
                'fake_context', fakes.DB_IMAGE_1, fakes.OSImage(os_image),
                None, None)

        self.assertEqual('instance-store', image['rootDeviceType'])
        self.assertNotIn('blockDeviceMapping', image)

        # check Glance status translation
        os_image = fakes.OSImage({'id': fakes.ID_OS_IMAGE_1})

        def check_status_translation(status, expected):
            os_image.status = status
            image = image_api._format_image(
                'fake_context', fakes.DB_IMAGE_1, os_image, None, None)
            self.assertEqual(expected, image['imageState'],
                             "Wrong '%s' Glance status translation" % status)
        check_status_translation('queued', 'pending')
        check_status_translation('saving', 'pending')
        check_status_translation('active', 'available')
        check_status_translation('killed', 'deregistered')
        check_status_translation('pending_delete', 'deregistered')
        check_status_translation('deleted', 'deregistered')
        check_status_translation('deactivated', 'invalid')
        check_status_translation('unknown-status', 'error')

        # check internal state translation
        os_image.status = 'queued'

        def check_state_translation(state, expected):
            os_image.image_state = state
            image = image_api._format_image(
                'fake_context', fakes.DB_IMAGE_1, os_image, None, None)
            self.assertEqual(expected, image['imageState'],
                             "Wrong '%s' internal state translation" % state)

        for state in ('downloading', 'decrypting', 'untarring', 'uploading'):
            check_state_translation(state, 'pending')
        for state in ('failed_download', 'failed_decrypt', 'failed_untar',
                      'failed_upload'):
            check_state_translation(state, 'failed')
        os_image.status = 'active'
        check_state_translation('available', 'available')
        check_state_translation('unknown-state', 'available')

    def test_format_mappings(self):
        db_api = self.mock_db()
        # check virtual mapping formatting
        properties = {
            'mappings': [
                {'virtual': 'ami', 'device': '/dev/sda'},
                {'virtual': 'root', 'device': 'sda'},
                {'virtual': 'ephemeral0', 'device': 'sdb'},
                {'virtual': 'swap', 'device': 'sdc'},
                {'virtual': 'ephemeral1', 'device': 'sdd'},
                {'virtual': 'ephemeral2', 'device': 'sde'},
                {'virtual': 'ephemeral', 'device': 'sdf'},
                {'virtual': '/dev/sdf1', 'device': 'root'}],
        }
        expected = [
            {'virtualName': 'ephemeral0', 'deviceName': '/dev/sdb'},
            {'virtualName': 'swap', 'deviceName': '/dev/sdc'},
            {'virtualName': 'ephemeral1', 'deviceName': '/dev/sdd'},
            {'virtualName': 'ephemeral2', 'deviceName': '/dev/sde'},
        ]

        result = image_api._format_mappings('fake_context', properties)
        self.assertEqual(expected, result)

        # check bdm v2 formatting
        db_api.set_mock_items(fakes.DB_IMAGE_2, fakes.DB_VOLUME_3)
        properties = {
            'bdm_v2': True,
            'block_device_mapping': [
                {'boot_index': 0,
                 'snapshot_id': fakes.ID_OS_SNAPSHOT_1,
                 'source_type': 'snapshot',
                 'destination_type': 'volume'},
                {'boot_index': None,
                 'snapshot_id': fakes.ID_OS_SNAPSHOT_2,
                 'source_type': 'snapshot',
                 'destination_type': 'volume'},
                {'device_name': 'vdi',
                 'boot_index': -1,
                 'image_id': fakes.ID_OS_IMAGE_2,
                 'source_type': 'image',
                 'destination_type': 'volume',
                 'volume_size': 20},
                {'device_name': 'vdv',
                 'boot_index': -1,
                 'volume_id': fakes.ID_OS_VOLUME_3,
                 'source_type': 'volume',
                 'destination_type': 'volume'},
                {'device_name': 'vdb',
                 'boot_index': -1,
                 'source_type': 'blank',
                 'destination_type': 'volume',
                 'volume_size': 100,
                 'delete_on_termination': True},
            ],
        }
        expected = [
            {'deviceName': 'vdx',
             'ebs': {'snapshotId': fakes.ID_EC2_SNAPSHOT_1,
                     'deleteOnTermination': False}},
            {'ebs': {'snapshotId': fakes.ID_EC2_SNAPSHOT_2,
                     'deleteOnTermination': False}},
            {'deviceName': 'vdi',
             'ebs': {'snapshotId': fakes.ID_EC2_IMAGE_2,
                     'volumeSize': 20,
                     'deleteOnTermination': False}},
            {'deviceName': 'vdv',
             'ebs': {'snapshotId': fakes.ID_EC2_VOLUME_3,
                     'deleteOnTermination': False}},
            {'deviceName': 'vdb',
             'ebs': {'volumeSize': 100,
                     'deleteOnTermination': True}},
        ]
        result = image_api._format_mappings(
            'fake_context', properties, root_device_name='vdx',
            snapshot_ids={fakes.ID_OS_SNAPSHOT_1: fakes.ID_EC2_SNAPSHOT_1,
                          fakes.ID_OS_SNAPSHOT_2: fakes.ID_EC2_SNAPSHOT_2})
        self.assertEqual(expected, result)

        # check inheritance and generation of virtual name
        properties = {
            'mappings': [
                {'device': 'vdd', 'virtual': 'ephemeral1'},
            ],
            'bdm_v2': True,
            'block_device_mapping': [
                {'device_name': '/dev/vdb',
                 'source_type': 'blank',
                 'destination_type': 'local',
                 'guest_format': 'swap'},
                {'device_name': 'vdc',
                 'source_type': 'blank',
                 'destination_type': 'local',
                 'volume_size': 5},
                {'device_name': 'vde',
                 'source_type': 'blank',
                 'destination_type': 'local'},
            ],
        }
        expected = [
            {'deviceName': '/dev/vdd', 'virtualName': 'ephemeral1'},
            {'deviceName': '/dev/vdb', 'virtualName': 'swap'},
            {'deviceName': 'vdc', 'virtualName': 'ephemeral0'},
            {'deviceName': 'vde', 'virtualName': 'ephemeral2'},
        ]
        result = image_api._format_mappings('fake_context', properties)
        self.assertEqual(expected, result)

    def test_get_db_items(self):
        describer = image_api.ImageDescriber()
        describer.context = base.create_context()

        # NOTE(ft): the first requested image appears is user owend and public,
        # the second is absent
        db_api = self.mock_db()
        db_api.set_mock_items(fakes.DB_IMAGE_1)

        describer.ids = set([fakes.ID_EC2_IMAGE_1, fakes.ID_EC2_IMAGE_2])
        self.assertRaises(exception.InvalidAMIIDNotFound,
                          describer.get_db_items)

    def test_describe_images_being_created(self):
        db_api = self.mock_db()
        glance = self.mock_glance()
        context = base.create_context()
        image_id = fakes.random_ec2_id('ami')
        image = {'id': image_id,
                 'os_id': None,
                 'is_public': False,
                 'description': 'fake desc'}
        db_api.set_mock_items(image)
        db_api.get_public_items.return_value = []

        # describe cases when no glance image exists
        glance.images.list.return_value = []
        expected = {'imagesSet': [{'imageId': image_id,
                                   'description': 'fake desc',
                                   'imageOwnerId': fakes.ID_OS_PROJECT,
                                   'imageState': 'pending',
                                   'imageType': 'machine',
                                   'isPublic': False}]}

        # describe all images
        result = image_api.describe_images(context)
        self.assertEqual(expected, result)

        # describe the image
        result = image_api.describe_images(context, image_id=[image_id])
        self.assertEqual(expected, result)

        # describe with filter
        result = image_api.describe_images(
            context, filter=[{'name': 'name', 'value': 'noname'}])
        self.assertEqual({'imagesSet': []}, result)

        # describe failed image
        image['state'] = 'failed'
        expected['imagesSet'][0]['imageState'] = 'failed'
        result = image_api.describe_images(base.create_context())
        self.assertEqual(expected, result)

        # describe cases when glance image exists, db item is yet not updated
        del image['state']
        os_image_id = fakes.random_os_id()
        os_image = {'id': os_image_id,
                    'owner': fakes.ID_OS_PROJECT,
                    'status': 'active',
                    'visibility': 'private',
                    'ec2_id': image_id}
        glance.images.list.return_value = [fakes.OSImage(os_image)]
        expected['imagesSet'] = [{
            'architecture': None,
            'creationDate': None,
            'description': 'fake desc',
            'imageId': image_id,
            'imageLocation': 'None (None)',
            'imageOwnerId': fakes.ID_OS_PROJECT,
            'imageState': 'available',
            'imageType': 'machine',
            'isPublic': False,
            'name': None,
            'rootDeviceType': 'instance-store'}]

        # describe all images
        result = image_api.describe_images(context)
        self.assertEqual(expected, result)
        db_api.update_item.assert_called_once_with(
            context, tools.update_dict(image, {'os_id': os_image_id}))

        # describe the image
        db_api.reset_mock()
        result = image_api.describe_images(context, image_id=[image_id])
        self.assertEqual(expected, result)
        db_api.update_item.assert_called_once_with(
            context, tools.update_dict(image, {'os_id': os_image_id}))


class S3TestCase(base.BaseTestCase):

    def test_s3_parse_manifest(self):
        db_api = self.mock_db()
        glance = self.mock_glance()
        db_api.set_mock_items(fakes.DB_IMAGE_AKI_1, fakes.DB_IMAGE_ARI_1)
        glance.images.get.side_effect = (
            tools.get_by_1st_arg_getter({
                fakes.ID_OS_IMAGE_AKI_1: fakes.OSImage(fakes.OS_IMAGE_AKI_1),
                fakes.ID_OS_IMAGE_ARI_1: fakes.OSImage(fakes.OS_IMAGE_ARI_1)}))

        metadata, image_parts, key, iv = image_api._s3_parse_manifest(
            base.create_context(), AMI_MANIFEST_XML)

        expected_metadata = {
            'disk_format': 'ami',
            'container_format': 'ami',
            'architecture': 'x86_64',
            'kernel_id': fakes.ID_OS_IMAGE_AKI_1,
            'ramdisk_id': fakes.ID_OS_IMAGE_ARI_1,
            'mappings': [
                {"device": "sda1", "virtual": "ami"},
                {"device": "/dev/sda1", "virtual": "root"},
                {"device": "sda2", "virtual": "ephemeral0"},
                {"device": "sda3", "virtual": "swap"}]}
        self.assertThat(metadata,
                        matchers.DictMatches(expected_metadata,
                                             orderless_lists=True))
        self.assertThat(image_parts,
                        matchers.ListMatches(['foo']))
        self.assertEqual('foo', key)
        self.assertEqual('foo', iv)
        db_api.get_items_ids.assert_any_call(
            mock.ANY, 'aki', item_ids=(fakes.ID_EC2_IMAGE_AKI_1,),
            item_os_ids=None)
        db_api.get_items_ids.assert_any_call(
            mock.ANY, 'ari', item_ids=(fakes.ID_EC2_IMAGE_ARI_1,),
            item_os_ids=None)

    def test_s3_create_image_locations(self):
        self.configure(image_decryption_dir=None)
        glance = self.mock_glance()
        _handle, tempf = tempfile.mkstemp()
        fake_context = base.create_context()

        @mock.patch('ec2api.api.image._s3_untarzip_image')
        @mock.patch('ec2api.api.image._s3_decrypt_image')
        @mock.patch('ec2api.api.image._s3_download_file')
        @mock.patch('ec2api.api.image._s3_conn')
        def do_test(s3_conn, s3_download_file, s3_decrypt_image,
                    s3_untarzip_image):
            (s3_conn.return_value.
             get_object.return_value) = {'Body': FILE_MANIFEST_XML}
            s3_download_file.return_value = tempf
            s3_untarzip_image.return_value = tempf
            os_image_id = fakes.random_os_id()
            (glance.images.create.return_value) = (
                fakes.OSImage({'id': os_image_id,
                               'status': 'queued'}))

            data = [
                ({'image_location': 'testbucket_1/test.img.manifest.xml'},
                 'testbucket_1', 'test.img.manifest.xml'),
                ({'image_location': '/testbucket_2/test.img.manifest.xml'},
                 'testbucket_2', 'test.img.manifest.xml')]
            for mdata, bucket, manifest in data:
                image = image_api._s3_create(fake_context, mdata)
                eventlet.sleep()
                self.glance.images.update.assert_called_with(
                    os_image_id, image_state='available')
                self.glance.images.upload.assert_any_call(
                    os_image_id, mock.ANY)
                s3_conn.return_value.get_object.assert_called_with(
                    Bucket=bucket, Key=manifest)
                s3_download_file.assert_called_with(
                    mock.ANY, bucket, 'foo', mock.ANY)
                s3_decrypt_image.assert_called_with(
                    fake_context, mock.ANY, 'foo', 'foo', mock.ANY)
                s3_untarzip_image.assert_called_with(mock.ANY, mock.ANY)

            do_test()

    @mock.patch('ec2api.api.image.eventlet.spawn_n')
    def test_s3_create_bdm(self, spawn_n):
        glance = self.mock_glance()
        metadata = {'image_location': 'fake_bucket/fake_manifest',
                    'root_device_name': '/dev/sda1',
                    'block_device_mapping': [
                        {'device_name': '/dev/sda1',
                         'snapshot_id': fakes.ID_OS_SNAPSHOT_1,
                         'delete_on_termination': True},
                        {'device_name': '/dev/sda2',
                         'virtual_name': 'ephemeral0'},
                        {'device_name': '/dev/sdb0',
                         'no_device': True}]}
        fake_context = base.create_context()
        with mock.patch('ec2api.api.image._s3_conn') as s3_conn:

            (s3_conn.return_value.
             get_object.return_value) = {'Body': FILE_MANIFEST_XML}

            image_api._s3_create(fake_context, metadata)

            glance.images.create.assert_called_once_with(
                disk_format='ami', container_format='ami',
                visibility='private', architecture='x86_64',
                image_state='pending', root_device_name='/dev/sda1',
                block_device_mapping=[{'device_name': '/dev/sda1',
                                       'snapshot_id': fakes.ID_OS_SNAPSHOT_1,
                                       'delete_on_termination': True},
                                      {'device_name': '/dev/sda2',
                                       'virtual_name': 'ephemeral0'},
                                      {'device_name': '/dev/sdb0',
                                       'no_device': True}],
                image_location='fake_bucket/fake_manifest')

    def test_s3_malicious_tarballs(self):
        self.assertRaises(
            exception.EC2InvalidException,
            image_api._s3_test_for_malicious_tarball,
            "/unused", os.path.join(os.path.dirname(__file__), 'abs.tar.gz'))
        self.assertRaises(
            exception.EC2InvalidException,
            image_api._s3_test_for_malicious_tarball,
            "/unused", os.path.join(os.path.dirname(__file__), 'rel.tar.gz'))

    def test_decrypt_text(self):
        public_key = os.path.join(os.path.dirname(__file__), 'test_cert.pem')
        private_key = os.path.join(os.path.dirname(__file__),
                                   'test_private_key.pem')
        subject = "/C=RU/ST=Moscow/L=Moscow/O=Progmatic/CN=RootCA"
        certificate_file = processutils.execute('openssl',
                                                'req', '-x509', '-new',
                                                '-key', private_key,
                                                '-days', '365',
                                                '-out', public_key,
                                                '-subj', subject)
        text = "some @#!%^* test text"
        process_input = text.encode("ascii") if six.PY3 else text
        enc, _err = processutils.execute('openssl',
                                         'rsautl',
                                         '-certin',
                                         '-encrypt',
                                         '-inkey', public_key,
                                         process_input=process_input,
                                         binary=True)
        self.assertRaises(exception.EC2Exception, image_api._decrypt_text, enc)
        self.configure(x509_root_private_key=private_key)
        dec = image_api._decrypt_text(enc)
        self.assertIsInstance(dec, bytes)
        if six.PY3:
            dec = dec.decode('ascii')
        self.assertEqual(text, dec)



# Copyright 2014
# The Cloudscaling Group, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest import mock

from oslotest import base as test_base

from ec2api.api import common


class OnCrashCleanerTestCase(test_base.BaseTestCase):
    class FakeException(Exception):
        pass

    def test_clean(self):
        obj = mock.MagicMock()

        def run():
            with common.OnCrashCleaner() as cleaner:
                cleaner.addCleanup(obj.fake_clean_method,
                                   555, 'arg', {'k': 'v'})
                cleaner.addCleanup(obj.fake_clean_method,
                                   666, 'param', {'key': 'value'})
                raise self.FakeException()

        self.assertRaises(self.FakeException, run)
        self.assertEqual([mock.call(666, 'param', {'key': 'value'}),
                          mock.call(555, 'arg', {'k': 'v'})],
                         obj.fake_clean_method.mock_calls)

    @mock.patch.object(common, 'LOG')
    def test_approve_partially(self, log):
        class FakeCrasherClass(object):
            call_count = 0

            def fake_crashed_clean_method(self, *args, **kwargs):
                self.call_count += 1
                raise Exception()

            def __call__(self):
                raise Exception()

        obj = mock.MagicMock()
        cls = FakeCrasherClass()

        with common.OnCrashCleaner() as cleaner:
            cleaner.addCleanup(obj.fake_clean_method_25),
            cleaner.addCleanup(obj.fake_clean_method)
            cleaner.addCleanup(cls.fake_crashed_clean_method)
            cleaner.approveChanges()
            cleaner.addCleanup(cls)
            cleaner.addCleanup(fake_standalone_crashed_clean_method)
            cleaner.addCleanup(cls.fake_crashed_clean_method,
                               'args', 666, {'key': 'value'},
                               s='args', i=666, d={'key': 'value'})
            cleaner.addCleanup(obj.fake_clean_method, 'params')
            raise Exception()

        self.assertEqual(1, cls.call_count)
        self.assertEqual(3, log.warning.call_count)
        self.assertIn('ec2api.tests.unit.test_common.FakeCrasherClass.'
                      'fake_crashed_clean_method',
                      log.warning.mock_calls[0][1][0])
        for arg in ["'args'", "666", "{'key': 'value'}",
                    "s='args'", "i=666", "d={'key': 'value'}"]:
            self.assertIn(arg, log.warning.mock_calls[0][1][0])
        self.assertIn('ec2api.tests.unit.test_common.'
                      'fake_standalone_crashed_clean_method',
                      log.warning.mock_calls[1][1][0])
        self.assertIn('ec2api.tests.unit.test_common.FakeCrasherClass',
                      log.warning.mock_calls[2][1][0])
        obj.fake_clean_method.assert_called_once_with('params')
        self.assertFalse(obj.fake_clean_method_25.called)

    def test_normal_flow(self):
        obj = mock.MagicMock()

        with common.OnCrashCleaner() as cleaner:
            cleaner.addCleanup(obj.fake_clean_method),
            cleaner.addCleanup(obj.fake_clean_method_25),

        self.assertFalse(obj.fake_clean_method.called)
        self.assertFalse(obj.fake_clean_method_25.called)

    def test_filter(self):
        obj = common.UniversalDescriber()
        obj.FILTER_MAP = {'prop1': 'prop-1', 'prop2': 'prop-2'}

        res = obj.filtered_out(
            {'prop-1': 'val-0', 'prop-2': 'val-123'},
            [{'name': 'prop1', 'value': ['val-0']}])
        self.assertFalse(res)

        res = obj.filtered_out(
            {'prop-1': 'val-0', 'prop-2': 'val-123'},
            [{'name': 'prop1', 'value': ['val-0', '0-val']}])
        self.assertFalse(res)

        res = obj.filtered_out(
            {'prop-1': 'val-0', 'prop-2': 'val-123'},
            [{'name': 'prop1', 'value': ['0-val', 'val-0']}])
        self.assertFalse(res)

        res = obj.filtered_out(
            {'prop-1': 'val-0', 'prop-2': 'val-123'},
            [{'name': 'prop1', 'value': ['0-val']}])
        self.assertTrue(res)

        res = obj.filtered_out(
            {'prop-1': 'val-0', 'prop-2': 'val-123'},
            [{'name': 'prop1', 'value': ['val-0']},
             {'name': 'prop2', 'value': ['val-123']}])
        self.assertFalse(res)

        res = obj.filtered_out(
            {'prop-1': 'val-0', 'prop-2': 'val-123'},
            [{'name': 'prop1', 'value': ['val-0']},
             {'name': 'prop2', 'value': ['123-val']}])
        self.assertTrue(res)

        res = obj.filtered_out(
            {'prop-1': 'val-0', 'prop-2': 'val-123'},
            [{'name': 'prop1', 'value': ['0-val']},
             {'name': 'prop2', 'value': ['val-123']}])
        self.assertTrue(res)


def fake_standalone_crashed_clean_method():
    raise Exception()
